'''
Tests for the vSQL binary bitwise "or" operator ``A | B``.

To run the tests, :mod:`pytest` is required.
'''

import datetime
from conftest import *


###
### Tests
###

def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_bool | False", where="r.identifier == 'bool_false'") == 0


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool | False", where="r.identifier == 'bool_true'") == 1


def test_bool_int1(db, vsql_data):
	assert expr(db, "r.v_bool | 0", where="r.identifier == 'bool_false'") == 0


def test_bool_int2(db, vsql_data):
	assert expr(db, "r.v_bool | 3", where="r.identifier == 'bool_true'") == 3


def test_int_bool1(db, vsql_data):
	assert expr(db, "r.v_int | False", where="r.identifier == 'int'") == 1776


def test_int_bool2(db, vsql_data):
	assert expr(db, "r.v_int | True", where="r.identifier == 'int'") == 1777


def test_int_int1(db, vsql_data):
	assert expr(db, "r.v_int | 313", where="r.identifier == 'int'") == 2041


def test_int_int2(db, vsql_data):
	assert expr(db, "r.v_int | 0", where="r.identifier == 'int'") == 1776


def test_int_int3(db, vsql_data):
	assert expr(db, "(-r.v_int) | 313", where="r.identifier == 'int'") == -1735


def test_intset_intset1(db, vsql_data):
	assert set(expr(db, "{1} | {1}")) == {1}


def test_intset_intset2(db, vsql_data):
	assert set(expr(db, "{1} | {2}")) == {1, 2}


def test_numberset_numberset1(db, vsql_data):
	assert set(expr(db, "{1.1} | {2.2}")) == {1.1, 2.2}


def test_numberset_numberset2(db, vsql_data):
	assert set(expr(db, "{1.1, 2.2} | {2.2, 3.3}")) == {1.1, 2.2, 3.3}


def test_strset_strset1(db, vsql_data):
	assert set(expr(db, '{"gurk"} | {"gurk"}')) == {"gurk"}


def test_strset_strset2(db, vsql_data):
	assert set(expr(db, '{"gurk", "hurz"} | {"hinz", "kunz"}')) == {"gurk", "hurz", "hinz", "kunz"}


def test_dateset_dateset1(db, vsql_data):
	assert set(expr(db, "{@(2000-02-29)} | {@(2000-03-01)}")) == {datetime.datetime(2000, 2, 29), datetime.datetime(2000, 3, 1)}


def test_dateset_dateset2(db, vsql_data):
	assert set(expr(db, "{@(2000-02-29), @(2000-03-01)} | {@(2000-03-01), @(2000-03-02)}")) == {datetime.datetime(2000, 2, 29), datetime.datetime(2000, 3, 1), datetime.datetime(2000, 3, 2)}


def test_datetimeset_datetimeset1(db, vsql_data):
	assert set(expr(db, "{@(2000-02-29T12:34:56)} | {@(2000-03-01T12:34:56)}")) == {datetime.datetime(2000, 2, 29, 12, 34, 56), datetime.datetime(2000, 3, 1, 12, 34, 56)}


def test_datetimeset_datetimeset2(db, vsql_data):
	assert set(expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} | {@(2000-03-01T12:34:56), @(2000-03-02T12:34:56)}")) == {datetime.datetime(2000, 2, 29, 12, 34, 56), datetime.datetime(2000, 3, 1, 12, 34, 56), datetime.datetime(2000, 3, 2, 12, 34, 56)}


"""
Tests for the vSQL binary item access operator ``A[B]``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *
import datetime

###
### Tests
###

def test_str_bool(db, vsql_data):
	assert expr(db, "r.v_str[True]", where="r.identifier == 'str'") == 'u'


def test_str_int(db, vsql_data):
	assert expr(db, "r.v_str[2]", where="r.identifier == 'str'") == 'r'


def test_clob_bool(db, vsql_data):
	assert expr(db, "r.v_clob[True]", where="r.identifier == 'clob'") == 'u'


def test_clob_int(db, vsql_data):
	assert expr(db, "r.v_clob[99998]", where="r.identifier == 'clob'") == 'r'


def test_strlist_bool(db, vsql_data):
	assert expr(db, "['gurk', 'hurz', 'hinz', 'kunz'][True]") == 'hurz'


def test_strlist_int(db, vsql_data):
	assert expr(db, "['gurk', 'hurz', 'hinz', 'kunz'][2]") == 'hinz'


def test_intlist_bool(db, vsql_data):
	assert expr(db, "[1, 2, 3][True]") == 2


def test_intlist_int(db, vsql_data):
	assert expr(db, "[1, 2, 3][2]") == 3


def test_numberlist_bool(db, vsql_data):
	assert expr(db, "[1.1, 2.2, 3.3][True]") == 2.2


def test_numberlist_int(db, vsql_data):
	assert expr(db, "[1.1, 2.2, 3.3][2]") == 3.3


def test_datelist_bool(db, vsql_data):
	assert expr(db, "[@(2000-02-29), @(2000-03-01), @(2000-03-02)][True]") == datetime.datetime(2000, 3, 1)


def test_datelist_int(db, vsql_data):
	assert expr(db, "[@(2000-02-29), @(2000-03-01), @(2000-03-02)][2]") == datetime.datetime(2000, 3, 2)


def test_datetimelist_bool(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56), @(2000-03-02T12:34:56)][True]") == datetime.datetime(2000, 3, 1, 12, 34, 56)


def test_datetimelist_int(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56), @(2000-03-02T12:34:56)][2]") == datetime.datetime(2000, 3, 2, 12, 34, 56)


def test_nulllist_bool1(db, vsql_data):
	assert expr(db, "[][False]") is None


def test_nulllist_bool2(db, vsql_data):
	assert expr(db, "[None, None][True]") is None


def test_nulllist_int1(db, vsql_data):
	assert expr(db, "[][0]") is None


def test_nulllist_int2(db, vsql_data):
	assert expr(db, "[None, None][1]") is None


"""
Tests for :class:`ll.vsql.Query`

To run the tests, :mod:`pytest` is required.
"""

import math, datetime

from conftest import *

from ll import vsql


field_table = vsql.Group("vsql_field")
field_table.add_field("id", vsql.DataType.STR, "{a}.fld_id")
field_table.add_field("name", vsql.DataType.STR, "{a}.fld_name")
field_table.add_field("parent", vsql.DataType.STR, "{a}.fld_id_super", "{m}.fld_id_super = {d}.fld_id", field_table)


person_table = vsql.Group("vsql_person")
person_table.add_field("id", vsql.DataType.STR, "{a}.per_id")
person_table.add_field("firstname", vsql.DataType.STR, "{a}.per_firstname")
person_table.add_field("lastname", vsql.DataType.STR, "{a}.per_lastname")
person_table.add_field("gender", vsql.DataType.STR, "{a}.per_gender")
person_table.add_field("field", vsql.DataType.STR, "{a}.fld_id", "{m}.fld_id = {d}.fld_id", field_table)
person_table.add_field("date_of_birth", vsql.DataType.DATE, "{a}.per_date_of_birth")
person_table.add_field("date_of_death", vsql.DataType.DATE, "{a}.per_date_of_death")
person_table.add_field("country_of_birth", vsql.DataType.STR, "{a}.per_country_of_birth")
person_table.add_field("grave", vsql.DataType.GEO, "{a}.per_grave")
person_table.add_field("nobel_prize", vsql.DataType.BOOL, "{a}.per_nobel_prize")
person_table.add_field("url", vsql.DataType.STR, "{a}.per_url")
person_table.add_field("createdat", vsql.DataType.DATETIME, "{a}.per_createdat")

p = vsql.Field("p", vsql.DataType.STR, "1 = 1", "2 = 2", refgroup=person_table)


###
### Tests
###

def test_query_comment(db, vsql_data):
	q = vsql.Query("foo")
	assert "/* foo */" in q.sqlsource()


def test_query_badcomment(db, vsql_data):
	q = vsql.Query("/* foo */")
	assert q.sqlsource().count("*/") == 1


def test_query_simple(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.select_vsql("p.firstname", alias="fn")
	q.where_vsql("p.lastname == 'Einstein'")
	rs = execute(db, q)

	assert rs[0].fn == "Albert"


def test_query_foreignkey(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.select_vsql("p.field.parent.name", alias="fld")
	q.where_vsql("p.lastname == 'Einstein'")
	rs = execute(db, q)

	assert rs[0].fld == "Science"


def test_query_count_all(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.aggregate_vsql("count()", "Number of persons", "c")
	rs = execute(db, q)

	assert rs[0].c == 10


def test_query_count_by_gender(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.aggregate_vsql("group(p.gender)")
	q.aggregate_vsql("count()")
	rs = execute(db, q)

	assert {r[0]: r[1] for r in rs} == {"f": 3, "m": 7}


def test_query_oldest_by_gender(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.aggregate_vsql("group(p.gender)")
	q.aggregate_vsql("max( int( ( (p.date_of_death or @(2000-02-29)) - p.date_of_birth ).days / 365.2425 ) )")
	rs = execute(db, q)

	assert {r[0]: int(r[1]) for r in rs} == {
		"f": 66, # Marie Curie
		"m": 93, # Ronald Reagan
	}


def test_query_count_by_field(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.aggregate_vsql("group(p.field.id)")
	q.aggregate_vsql("count()")
	rs = execute(db, q)

	assert {r[0]: int(r[1]) for r in rs} == {
		"computerscience": 1, # Donald Kunth
		"film": 1, # Marilyn Monroe
		"mathematics": 2, # Bern Reieman, Carl Friedrich Gauß
		"music": 1, # Elvis Presley
		"physics": 2, # Albert Einstain, Marie Curie
		"politics": 2, # Ronald Reagan, Angela Merkel
		"sport": 1, # Muhammad Ali
	}


def test_query_first_last_by_century(db, vsql_data):
	q = vsql.Query(p=p)
	q.from_vsql("p")
	q.aggregate_vsql("group(p.date_of_birth.year//100)")
	q.aggregate_vsql("min(str(p.date_of_birth))")
	q.aggregate_vsql("max(str(p.date_of_birth))")
	q.aggregate_vsql("count()")
	rs = execute(db, q)

	assert {r[0]: (r[1], r[2], r[3]) for r in rs} == {
		17: (
			'1777-04-30',
			'1777-04-30',
			1,
		),
		18: (
			'1826-06-17',
			'1879-03-14',
			3,
		),
		19: (
			'1911-02-06',
			'1954-06-17',
			6,
		),
	}


def test_query_sql(db, vsql_data):
	q = vsql.Query()
	q.select_sql("count(*)")
	q.from_sql("vsql_person")
	rs = [list(r) for r in execute(db, q)]

	assert rs == [[10]]



"""
Tests for the vSQL unary bitwise not operator ``~``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import expr


###
### Tests
###

def test_bool1(db, vsql_data):
	assert expr(db, "~r.v_bool", where="r.identifier == 'none'") is None


def test_bool2(db, vsql_data):
	assert expr(db, "~r.v_bool", where="r.identifier == 'bool_false'") == -1


def test_bool3(db, vsql_data):
	assert expr(db, "~r.v_bool", where="r.identifier == 'bool_true'") == -2


def test_int1(db, vsql_data):
	assert expr(db, "~r.v_int", where="r.identifier == 'none'") is None


def test_int2(db, vsql_data):
	assert expr(db, "~r.v_int", where="r.identifier == 'int'") == -1777


'''
Tests for the vSQL binary bitwise "and" operator ``A & B``.

To run the tests, :mod:`pytest` is required.
'''

import datetime
from conftest import *


###
### Tests
###

def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_bool & True", where="r.identifier == 'bool_false'") == 0


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool & True", where="r.identifier == 'bool_true'") == 1


def test_bool_int1(db, vsql_data):
	assert expr(db, "r.v_bool & 3", where="r.identifier == 'bool_false'") == 0


def test_bool_int2(db, vsql_data):
	assert expr(db, "r.v_bool & 3", where="r.identifier == 'bool_true'") == 1


def test_int_bool1(db, vsql_data):
	assert expr(db, "r.v_int & False", where="r.identifier == 'int'") == 0


def test_int_bool2(db, vsql_data):
	assert expr(db, "r.v_int & True", where="r.identifier == 'int'") == 0


def test_int_int1(db, vsql_data):
	assert expr(db, "r.v_int & 313", where="r.identifier == 'int'") == 48


def test_int_int2(db, vsql_data):
	assert expr(db, "r.v_int & 270", where="r.identifier == 'int'") == 0


def test_int_int3(db, vsql_data):
	assert expr(db, "(-r.v_int) & 313", where="r.identifier == 'int'") == 272


def test_intset_intset1(db, vsql_data):
	assert expr(db, "{1} & {2}") == []


def test_intset_intset2(db, vsql_data):
	assert expr(db, "{1, 2} & {2, 3}") == [2]


def test_numberset_numberset1(db, vsql_data):
	assert expr(db, "{1.1} & {2.2}") == []


def test_numberset_numberset2(db, vsql_data):
	assert expr(db, "{1.1, 2.2} & {2.2, 3.3}") == [2.2]


def test_strset_strset1(db, vsql_data):
	assert expr(db, '{"gurk", "hurz"} & {"hinz", "kunz"}') == []


def test_strset_strset2(db, vsql_data):
	assert expr(db, '{"gurk", "hurz"} & {"hurz", "kunz"}') == ["hurz"]


def test_dateset_dateset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29), @(2000-03-01)} & {@(2000-03-02), @(2000-03-03)}") == []


def test_dateset_dateset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29), @(2000-03-01)} & {@(2000-03-01), @(2000-03-02)}") == [datetime.datetime(2000, 3, 1)]


def test_datetimeset_datetimeset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} & {@(2000-03-02T12:34:56), @(2000-03-03T12:34:56)}") == []


def test_datetimeset_datetimeset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} & {@(2000-03-01T12:34:56), @(2000-03-02T12:34:56)}") == [datetime.datetime(2000, 3, 1, 12, 34, 56)]


"""
Tests for vSQL literals.

To run the tests, :mod:`pytest` is required.
"""

###
### Tests
###

import datetime

from ll import vsql

from conftest import execute, expr, vsql_r


def test_literal_none(db, vsql_data):
	assert expr(db, "None") is None


def test_literal_bool(db, vsql_data):
	assert expr(db, "False") == 0
	assert expr(db, "True") == 1


def test_literal_int(db, vsql_data):
	assert expr(db, str(42)) == 42
	assert expr(db, bin(42)) == 42
	assert expr(db, oct(42)) == 42
	assert expr(db, hex(42)) == 42
	assert expr(db, str(-42)) == -42
	assert expr(db, bin(-42)) == -42
	assert expr(db, oct(-42)) == -42
	assert expr(db, hex(-42)) == -42


def test_literal_float(db, vsql_data):
	assert expr(db, "42.5") == 42.5
	assert expr(db, "-42.5") == -42.5
	assert expr(db, "1e2") == 100.0
	assert expr(db, "-1e2") == -100.0


def test_literal_string(db, vsql_data):
	assert expr(db, "'foo'") == "foo"
	assert expr(db, "'\x01\xff\u3042'") == "\x01\xff\u3042"
	assert expr(db, "'\\a\\b\\t\\n\\f\\r\\\"\\'\\\\'") == "\a\b\t\n\f\r\"'\\"
	assert expr(db, "'\\x00\\xff\\u3042'") == "\x00\xff\u3042"
	assert expr(db, "'\\U0001f389'") == "🎉"
	# FIXME: This doesn't work yet, because UL4 inherits the 16-bit limitation of ANTLR 3
	# assert expr(db, "'🎉'") == "🎉"


def test_literal_date(db, vsql_data):
	assert expr(db, "@(2000-02-29)") == datetime.datetime(2000, 2, 29)
	assert expr(db, "@(2000-02-29T12:34:56)") == datetime.datetime(2000, 2, 29, 12, 34, 56)


def test_literal_color(db, vsql_data):
	assert expr(db, "#369") == 0x336699ff
	assert expr(db, "#123456") == 0x123456ff
	assert expr(db, "#369c") == 0x336699cc
	assert expr(db, "#12345678") == 0x12345678


def test_x(db, vsql_data):
	q = vsql.Query(r=vsql_r)
	q.select_vsql("r.identifier")
	q.where_vsql("r.identifier == 'none'")

	rs = execute(db, q)

	assert len(rs) == 1
	assert rs[0][0] == "none"



#! /usr/bin/env/python
# -*- coding: utf-8 -*-
# cython: language_level=3, always_allow_keywords=True

## Copyright 2009-2025 by LivingLogic AG, Bayreuth/Germany
## Copyright 2009-2025 by Walter Dörwald
##
## All Rights Reserved
##
## See ll/xist/__init__.py for the license


import math

import pytest

from ll import color


def test_constructor():
	assert color.Color(10, 20, 30) == (10, 20, 30, 255)
	assert color.Color(40, 50, 60, 70) == (40, 50, 60, 70)


def test_fromcss():
	assert color.Color.fromcss("red") == (0xff, 0x0, 0x0, 0xff)
	assert color.Color.fromcss("#123") == (0x11, 0x22, 0x33, 0xff)
	assert color.Color.fromcss("#123456") == (0x12, 0x34, 0x56, 0xff)
	assert color.Color.fromcss("#abcdef") == (0xab, 0xcd, 0xef, 0xff)
	assert color.Color.fromcss("#ABCDEF") == (0xab, 0xcd, 0xef, 0xff)
	assert color.Color.fromcss("rgb(12, 34, 56)") == (12, 34, 56, 0xff)
	assert color.Color.fromcss("rgb(20%, 40%, 60%)") == (0x33, 0x66, 0x99, 0xff)
	assert color.Color.fromcss("rgba(12, 34, 56, 0)") == (12, 34, 56, 0x0)
	assert color.Color.fromcss("rgba(12, 34, 56, 1)") == (12, 34, 56, 0xff)
	assert color.Color.fromcss("rgba(20%, 40%, 60%, 0)") == (0x33, 0x66, 0x99, 0x0)
	assert color.Color.fromcss("rgba(20%, 40%, 60%, 1)") == (0x33, 0x66, 0x99, 0xff)


def test_fromrgb():
	assert color.Color.fromrgb(0.2, 0.4, 0.6, 0.8) == color.Color(0x33, 0x66, 0x99, 0xcc)


def test_repr():
	assert repr(color.red) == "Color(0xff, 0x00, 0x00)"
	assert repr(color.Color(0x12, 0x34, 0x56, 0x78)) == "Color(0x12, 0x34, 0x56, 0x78)"


def test_str():
	assert str(color.Color(0x12, 0x34, 0x56)) == "#123456"
	assert str(color.Color(0x12, 0x34, 0x56, 0x78)) == "#12345678"


def test_r_g_b():
	c = color.Color(0x12, 0x34, 0x56, 0x78)
	assert c.r() == 0x12
	assert c.g() == 0x34
	assert c.b() == 0x56
	assert c.a() == 0x78


def test_hls():
	assert (0, 1, 0) == tuple(map(int, color.css("#fff").hls()))


def test_hlsa():
	assert (0, 1, 0, 1) == tuple(map(int, color.css("#fff").hlsa()))


def test_hsv():
	assert (0, 0, 1) == tuple(map(int, color.css("#fff").hsv()))


def test_hsva():
	assert (0, 0, 1, 1) == tuple(map(int, color.css("#fff").hsva()))


def test_hue():
	assert math.isclose(color.css("#f00").hue(), 0)
	assert math.isclose(color.css("#0f0").hue(), 120/360)
	assert math.isclose(color.css("#00f").hue(), 240/360)


def test_sat():
	assert math.isclose(0, color.css("#000").sat())
	assert math.isclose(0, color.css("#fff").sat())
	assert math.isclose(1, color.css("#f00").sat())
	assert math.isclose(1, color.css("#0f0").sat())
	assert math.isclose(1, color.css("#00f").sat())


def test_light():
	math.isclose(0, color.css("#000").light())
	math.isclose(1, color.css("#fff").light())


def test_lum():
	assert math.isclose(1.0   , color.css("#fff").lum())
	assert math.isclose(0.0   , color.css("#000").lum())
	assert math.isclose(0.2126, color.css("#f00").lum())
	assert math.isclose(0.7152, color.css("#0f0").lum())
	assert math.isclose(0.0722, color.css("#00f").lum())


def test_withhue():
	assert color.css("#f00") == color.css("#0f0").withhue(0/6)
	assert color.css("#0f0") == color.css("#f00").withhue(2/6)


def test_withlight():
	assert color.white == color.black.withlight(1.0)
	assert color.black == color.black.withlight(0.0)
	assert color.white == color.white.withlight(1.0)
	assert color.black == color.white.withlight(0.0)


def test_abslight():
	assert color.white == color.black.abslight(1.0)
	assert color.css('#333') == color.black.abslight(0.2)
	assert color.css('#666') == color.css('#333').abslight(0.2)
	assert color.black == color.black.abslight(0.0)
	assert color.black == color.black.abslight(-1.0)
	assert color.white == color.white.abslight(1.0)
	assert color.white == color.white.abslight(0.0)
	assert color.css('#ccc') == color.white.abslight(-0.2)
	assert color.black == color.white.abslight(-1.0)


def test_rellight():
	assert color.white == color.black.rellight(1.0)
	assert color.css('#333') == color.black.rellight(0.2)
	assert color.css('#999') == color.css('#333').rellight(0.5)
	assert color.black == color.black.rellight(0.0)
	assert color.black == color.black.rellight(-1.0)
	assert color.white == color.white.rellight(1.0)
	assert color.white == color.white.rellight(0.0)
	assert color.css('#ccc') == color.white.rellight(-0.2)
	assert color.css('#666') == color.css('#ccc').rellight(-0.5)
	assert color.black == color.white.rellight(-1.0)


def test_withsat():
	assert color.css("#7f7f7f") == color.css("#0f0").withsat(0.0)
	assert color.css("#0f0") == color.css("#0f0").withsat(1.0)


def test_witha():
	assert color.Color(0x00, 0x63, 0xa8, 0x2a) == color.Color(0x00, 0x63, 0xa8).witha(42)


def test_withlum():
	assert "#000" == str(color.css("#fff").withlum(0))
	assert "#333" == str(color.css("#fff").withlum(0.2))
	assert "#f00" == str(color.css("#f00").withlum(0.2126))
	assert "#0f0" == str(color.css("#0f0").withlum(0.7152))
	assert "#00f" == str(color.css("#00f").withlum(0.0722))


def test_abslum():
	assert "#fff" == str(color.css("#000").abslum(1))
	assert "#fff" == str(color.css("#fff").abslum(0))
	assert "#000" == str(color.css("#fff").abslum(-1))
	assert "#333" == str(color.css("#000").abslum(0.2))


def test_rellum():
	assert "#fff" == str(color.css("#000").rellum(1))
	assert "#fff" == str(color.css("#fff").rellum(0))
	assert "#000" == str(color.css("#fff").rellum(-1))
	assert "#888" == str(color.css("#888").rellum(0))
	assert "#f33" == str(color.css("#f00").rellum(0.2))
	assert "#3f3" == str(color.css("#0f0").rellum(0.2))
	assert "#33f" == str(color.css("#00f").rellum(0.2))


def test_invert():
	assert "#000" == str(color.css("#000").invert(0))
	assert "#333" == str(color.css("#000").invert(0.2))
	assert "#fff" == str(color.css("#000").invert(1))
	assert "#fff" == str(color.css("#000").invert())
	assert "#fff" == str(color.css("#fff").invert(0))
	assert "#ccc" == str(color.css("#fff").invert(0.2))
	assert "#000" == str(color.css("#fff").invert(1))
	assert "#000" == str(color.css("#fff").invert())
	assert "#0ff" == str(color.css("#f00").invert())
	assert "#f0f" == str(color.css("#0f0").invert())
	assert "#ff0" == str(color.css("#00f").invert())


def test_rgb():
	assert color.Color(0x33, 0x66, 0x99, 0xcc).rgb() == (0.2, 0.4, 0.6)


def test_rgba():
	assert color.Color(0x33, 0x66, 0x99, 0xcc).rgba() == (0.2, 0.4, 0.6, 0.8)


def test_combine():
	assert color.Color(0x12, 0x34, 0x56).combine(r=0x78) == (0x78, 0x34, 0x56, 0xff)
	assert color.Color(0x12, 0x34, 0x56).combine(g=0x78) == (0x12, 0x78, 0x56, 0xff)
	assert color.Color(0x12, 0x34, 0x56).combine(b=0x78) == (0x12, 0x34, 0x78, 0xff)
	assert color.Color(0x12, 0x34, 0x56).combine(a=0x78) == (0x12, 0x34, 0x56, 0x78)


def test_mul():
	assert 2*color.Color(0x12, 0x34, 0x56) == color.Color(0x24, 0x68, 0xac)
	assert color.Color(0x12, 0x34, 0x56)*2 == color.Color(0x24, 0x68, 0xac)


def test_truediv():
	assert color.Color(0x24, 0x68, 0xac)/2 == color.Color(0x12, 0x34, 0x56)


def test_floordiv():
	assert color.Color(0x25, 0x69, 0xad)//2 == color.Color(0x12, 0x34, 0x56)


def test_mod():
	assert color.Color(0x80, 0x80, 0x80) % color.Color(0xff, 0xff, 0xff) == color.Color(0x80, 0x80, 0x80)
	assert color.Color(0x80, 0x80, 0x80, 0x00) % color.Color(0xff, 0xff, 0xff) == color.Color(0xff, 0xff, 0xff)
	assert color.Color(0x80, 0x80, 0x80, 0x80) % color.Color(0xff, 0xff, 0xff) == color.Color(0xbf, 0xbf, 0xbf)


def test_css():
	assert color.Color(0x00, 0x00, 0x00) == color.css('black')
	assert color.Color(0xff, 0xff, 0xff) == color.css('white')
	assert color.Color(0x11, 0x22, 0x33) == color.css('#123')
	assert color.Color(0x11, 0x22, 0x33, 0x44) == color.css('#1234')
	assert color.Color(0x12, 0x34, 0x56) == color.css('#123456')
	assert color.Color(0x12, 0x34, 0x56, 0x78) == color.css('#12345678')
	assert color.Color(0x11, 0x33, 0x66) == color.css('rgb(17, 20%, 40%)')
	assert color.Color(0x11, 0x33, 0x66, 0x99) == color.css('rgba(17, 20%, 40%, 0.6)')
	assert color.Color(0x11, 0x33, 0x66, 0x99) == color.css('rgba(17, 20%, 40%, 60%)')
	with pytest.raises(ValueError):
		color.css('bad')
	assert color.Color(0x11, 0x22, 0x33) == color.css('bad', color.Color(0x11, 0x22, 0x33))


def test_mix():
	black = color.Color(0x00, 0x00, 0x00)
	white = color.Color(0xff, 0xff, 0xff)
	red = color.Color(0xff, 0x00, 0x00)
	green = color.Color(0x00, 0xff, 0x00)
	blue = color.Color(0x00, 0x00, 0xff)
	unblack = color.Color(0x00, 0x00, 0x00, 0x00)

	assert color.Color(0xaa, 0xaa, 0xaa) == color.mix(black, white, white)
	assert color.Color(0x55, 0x55, 0x55) == color.mix(black, black, white)
	assert color.Color(0xaa, 0xaa, 0xaa, 0xaa) == color.mix(unblack, white, white)
	assert color.Color(0xaa, 0xaa, 0xaa) == color.mix(black, 2, white)
	assert color.Color(0x11, 0x22, 0xcc) == color.mix(red, 2, green, 12, blue)



"""
Tests for vSQL methods.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###


def test_str_lower(db, vsql_data):
	assert expr(db, "'MISSISSIPPI'.lower()") == "mississippi"


def test_str_upper(db, vsql_data):
	assert expr(db, "'mississippi'.upper()") == "MISSISSIPPI"


def test_str_startswith(db, vsql_data):
	assert expr(db, "'mississippi'.startswith('missi')") == True


def test_str_endswith(db, vsql_data):
	assert expr(db, "'mississippi'.endswith('sippi')") == True


def test_str_strip1(db, vsql_data):
	assert expr(db, "'\\r\\t\\n foo \\r\\t\\n '.strip()") == "foo"


def test_str_strip2(db, vsql_data):
	assert expr(db, "'xyzzygurkxyzzy'.strip('xyz')") == "gurk"


def test_str_lstrip1(db, vsql_data):
	assert expr(db, "'\\r\\t\\n foo \\r\\t\\n '.lstrip()") == "foo \r\t\n "


def test_str_lstrip2(db, vsql_data):
	assert expr(db, "'xyzzygurkxyzzy'.lstrip('xyz')") == "gurkxyzzy"


def test_str_rstrip1(db, vsql_data):
	assert expr(db, "'\\r\\t\\n foo \\r\\t\\n '.rstrip()") == "\r\t\n foo"


def test_str_rstrip2(db, vsql_data):
	assert expr(db, "'xyzzygurkxyzzy'.rstrip('xyz')") == "xyzzygurk"


def test_str_find1(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('ks')") == -1


def test_str_find2(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk')") == 2


def test_str_find3(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', 2)") == 2


def test_str_find4(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', -3)") == 6


def test_str_find5(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', 2, 4)") == 2


def test_str_find6(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', 4, 8)") == 6


def test_str_find7(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('ur', -4, -1)") == 5


def test_str_find8(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', 2, 3)") == -1


def test_str_find9(db, vsql_data):
	assert expr(db, "'gurkgurk'.find('rk', 7)") == -1


def test_str_replace(db, vsql_data):
	assert expr(db, "'gurk'.replace('u', 'oo')") == "goork"


def test_str_split1(db, vsql_data):
	assert expr(db, "' \\t\\r\\nf \\t\\r\\no \\t\\r\\no \\t\\r\\n'.split()") == ['f', 'o', 'o']


def test_str_split2(db, vsql_data):
	assert expr(db, "' \\t\\r\\nf \\t\\r\\no \\t\\r\\no \\t\\r\\n'.split(None, 1)") == ['f', 'o \t\r\no']


def test_str_split3(db, vsql_data):
	assert expr(db, "'xxfxxoxxoxx'.split('xx')") == [None, 'f', 'o', 'o', None]


def test_str_split4(db, vsql_data):
	assert expr(db, "'xxfxxoxxoxx'.split('xx', 2)") == [None, 'f', 'oxxoxx']


def test_str_join_str(db, vsql_data):
	assert expr(db, "','.join('1234')") == "1,2,3,4"


def test_str_join_list(db, vsql_data):
	assert expr(db, "','.join(['1', '2', '3', '4'])") == "1,2,3,4"


def test_color_lum1(db, vsql_data):
	assert expr(db, "#000.lum()") == 0.0


def test_color_lum2(db, vsql_data):
	assert expr(db, "#fff.lum()") == 1.0


def test_date_week(db, vsql_data):
	assert expr(db, "@(2000-02-29).week()") == 9


def test_datetime_week(db, vsql_data):
	assert expr(db, "@(2000-02-29T12:34:56).week()") == 9



'''
Tests for the vSQL binary bitwise "exclusive or" operator ``A ^ B``.

To run the tests, :mod:`pytest` is required.
'''

from conftest import *


###
### Tests
###

def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_bool ^ False", where="r.identifier == 'bool_false'") == 0


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool ^ True", where="r.identifier == 'bool_false'") == 1


def test_bool_int(db, vsql_data):
	assert expr(db, "r.v_bool ^ 3", where="r.identifier == 'bool_true'") == 2


def test_int_bool(db, vsql_data):
	assert expr(db, "r.v_int ^ True", where="r.identifier == 'int'") == 1777


def test_int_int1(db, vsql_data):
	assert expr(db, "r.v_int ^ 313", where="r.identifier == 'int'") == 1993


def test_int_int2(db, vsql_data):
	assert expr(db, "(-r.v_int) ^ 313", where="r.identifier == 'int'") == -2007


"""
Tests for vSQL functions.

To run the tests, :mod:`pytest` is required.
"""

import math, datetime

from conftest import *


###
### Tests
###

def test_today(db, vsql_data):
	assert expr(db, "today()") >= datetime.datetime(2000, 2, 29)


def test_now(db, vsql_data):
	assert expr(db, "now()") >= datetime.datetime(2000, 2, 29, 12, 34, 56)


def test_bool(db, vsql_data):
	assert expr(db, "bool()") == False


def test_bool_none(db, vsql_data):
	assert expr(db, "bool(None)") == False


def test_bool_false(db, vsql_data):
	assert expr(db, "bool(False)") == False


def test_bool_true(db, vsql_data):
	assert expr(db, "bool(True)") == True


def test_bool_int_none(db, vsql_data):
	assert expr(db, "bool(r.v_int)", where="r.identifier == 'none'") == False


def test_bool_int_false(db, vsql_data):
	assert expr(db, "bool(0)") == False


def test_bool_int_true(db, vsql_data):
	assert expr(db, "bool(42)") == True


def test_bool_number_false(db, vsql_data):
	assert expr(db, "bool(0.0)") == False


def test_bool_number_true(db, vsql_data):
	assert expr(db, "bool(42.5)") == True


def test_bool_datedelta_false(db, vsql_data):
	assert expr(db, "bool(days(0))") == False


def test_bool_datedelta_true(db, vsql_data):
	assert expr(db, "bool(days(42))") == True


def test_bool_datetimedelta_false(db, vsql_data):
	assert expr(db, "bool(minutes(0))") == False


def test_bool_datetimedelta_true(db, vsql_data):
	assert expr(db, "bool(minutes(42))") == True


def test_bool_monthdelta_false(db, vsql_data):
	assert expr(db, "bool(monthdelta(0))") == False


def test_bool_monthdelta_true(db, vsql_data):
	assert expr(db, "bool(monthdelta(42))") == True


def test_bool_date(db, vsql_data):
	assert expr(db, "bool(@(2000-02-29))") == True


def test_bool_datetime(db, vsql_data):
	assert expr(db, "bool(@(2000-02-29T12:34:56))") == True


def test_bool_color(db, vsql_data):
	assert expr(db, "bool(#fff)") == True


def test_bool_str_false(db, vsql_data):
	assert expr(db, "bool('')") == False


def test_bool_str_true(db, vsql_data):
	assert expr(db, "bool('gurk')") == True


def test_bool_intlist(db, vsql_data):
	assert expr(db, "bool([42])") == True


def test_bool_numberlist(db, vsql_data):
	assert expr(db, "bool([42.5])") == True


def test_bool_strlist(db, vsql_data):
	assert expr(db, "bool(['gurk'])") == True


def test_bool_datelist(db, vsql_data):
	assert expr(db, "bool([today()])") == True


def test_bool_datetimelist(db, vsql_data):
	assert expr(db, "bool([now()])") == True


def test_bool_intset(db, vsql_data):
	assert expr(db, "bool({42})") == True


def test_bool_numberset(db, vsql_data):
	assert expr(db, "bool({42.5})") == True


def test_bool_strset(db, vsql_data):
	assert expr(db, "bool({'gurk'})") == True


def test_bool_dateset(db, vsql_data):
	assert expr(db, "bool({today()})") == True


def test_bool_datetimeset(db, vsql_data):
	assert expr(db, "bool({now()})") == True


def test_int(db, vsql_data):
	assert expr(db, "int()") == False


def test_int_bool_false(db, vsql_data):
	assert expr(db, "int(False)") == False


def test_int_bool_true(db, vsql_data):
	assert expr(db, "int(True)") == 1


def test_int_int(db, vsql_data):
	assert expr(db, "int(42)") == 42


def test_int_number(db, vsql_data):
	assert expr(db, "int(42.4)") == 42


def test_int_str_ok(db, vsql_data):
	assert expr(db, "int('42')") == 42


def test_int_str_bad(db, vsql_data):
	assert expr(db, "int('42.5')") is None


def test_int_str_very_bad(db, vsql_data):
	assert expr(db, "int('verybad')") is None


def test_float(db, vsql_data):
	assert expr(db, "float()") == 0.0


def test_float_bool_false(db, vsql_data):
	assert expr(db, "float(False)") == 0.0


def test_float_bool_true(db, vsql_data):
	assert expr(db, "float(True)") == 1.0


def test_float_int(db, vsql_data):
	assert expr(db, "float(42)") == 42.0


def test_float_number(db, vsql_data):
	assert expr(db, "float(42.5)") == 42.5


def test_float_str(db, vsql_data):
	assert expr(db, "float('42.5')") == 42.5


def test_float_str_bad(db, vsql_data):
	assert expr(db, "float('bad')") is None


def test_str(db, vsql_data):
	assert expr(db, "str()") is None


def test_str_bool_false(db, vsql_data):
	assert expr(db, "str(False)") == "False"


def test_str_bool_true(db, vsql_data):
	assert expr(db, "str(True)") == "True"


def test_str_int(db, vsql_data):
	assert expr(db, "str(-42)") == "-42"


def test_str_number1(db, vsql_data):
	assert expr(db, "str(42.0)") == "42.0"

def test_str_number2(db, vsql_data):
	assert expr(db, "str(-42.5)") == "-42.5"


def test_str_str(db, vsql_data):
	assert expr(db, "str('foo')") == "foo"


def test_str_date(db, vsql_data):
	assert expr(db, "str(@(2000-02-29))") == "2000-02-29"


def test_str_datetime(db, vsql_data):
	assert expr(db, "str(@(2000-02-29T12:34:56))") == "2000-02-29 12:34:56"


def test_str_datedelta_1(db, vsql_data):
	assert expr(db, "str(days(1))") == "1 day"


def test_str_datedelta_2(db, vsql_data):
	assert expr(db, "str(days(42))") == "42 days"


def test_str_datetimedelta_1(db, vsql_data):
	assert expr(db, "str(seconds(42))") == "0:00:42"


def test_str_datetimedelta_2(db, vsql_data):
	assert expr(db, "str(minutes(42))") == "0:42:00"


def test_str_datetimedelta_3(db, vsql_data):
	assert expr(db, "str(hours(17) + minutes(23))") == "17:23:00"


def test_str_datetimedelta_4(db, vsql_data):
	assert expr(db, "str(hours(42) + seconds(0))") == "1 day, 18:00:00"


def test_str_datetimedelta_5(db, vsql_data):
	assert expr(db, "str(days(42) + seconds(0))") == "42 days, 0:00:00"


def test_str_datetimedelta_6(db, vsql_data):
	assert expr(db, "str(days(42) + hours(17) + minutes(23))") == "42 days, 17:23:00"


def test_str_datetimedelta_7(db, vsql_data):
	assert expr(db, "str(-days(1) - hours(12) - minutes(34) - seconds(56))") == "-2 days, 11:25:04"


def test_str_monthdelta_1(db, vsql_data):
	assert expr(db, "str(monthdelta(0))") == "0 months"


def test_str_monthdelta_2(db, vsql_data):
	assert expr(db, "str(monthdelta(1))") == "1 month"


def test_str_monthdelta_3(db, vsql_data):
	assert expr(db, "str(monthdelta(42))") == "42 months"


def test_str_color_1(db, vsql_data):
	assert expr(db, "str(#000f)") == "#000"


def test_str_color_2(db, vsql_data):
	assert expr(db, "str(#fff0)") == "rgba(255, 255, 255, 0.000)"


def test_str_color_3(db, vsql_data):
	assert expr(db, "str(#123456)") == "#123456"


def test_str_color_4(db, vsql_data):
	assert expr(db, "str(#12345678)") == "rgba(18, 52, 86, 0.471)"


def test_str_geo_without_info(db, vsql_data):
	assert expr(db, "str(geo(49.95, 11.59))") == "<geo lat=49.95 long=11.59 info=None>"


def test_str_geo_with_info(db, vsql_data):
	assert expr(db, "str(geo(49.95, 11.59, 'Here'))") == "<geo lat=49.95 long=11.59 info='Here'>"


def test_str_intlist(db, vsql_data):
	assert expr(db, "str([1, 2, 3, None])") == "[1, 2, 3, None]"


def test_str_numberlist(db, vsql_data):
	assert expr(db, "str([1.2, 3.4, 5.6, None])") == "[1.2, 3.4, 5.6, None]"


def test_str_strlist(db, vsql_data):
	assert expr(db, "str(['foo', 'bar', None])") == "['foo', 'bar', None]"


def test_str_datelist(db, vsql_data):
	assert expr(db, "str([@(2000-02-29), None])") == "[@(2000-02-29), None]"


def test_str_datetimelist(db, vsql_data):
	assert expr(db, "str([@(2000-02-29T12:34:56), None])") == "[@(2000-02-29T12:34:56), None]"


# For the set test only include one non-``None`` value,
# as the order of the other elements is undefined

def test_str_intset(db, vsql_data):
	assert expr(db, "str({1, None})") == "{1, None}"


def test_str_numberset(db, vsql_data):
	assert expr(db, "str({1.2, None})") == "{1.2, None}"


def test_str_strset(db, vsql_data):
	assert expr(db, "str({'foo', None})") == "{'foo', None}"


def test_str_dateset(db, vsql_data):
	assert expr(db, "str({@(2000-02-29), None})") == "{@(2000-02-29), None}"


def test_str_datetimeset(db, vsql_data):
	assert expr(db, "str({@(2000-02-29T12:34:56), None})") == "{@(2000-02-29T12:34:56), None}"


def test_repr_none(db, vsql_data):
	assert expr(db, "repr(None)") == "None"


def test_repr_bool_false(db, vsql_data):
	assert expr(db, "repr(False)") == "False"


def test_repr_bool_True(db, vsql_data):
	assert expr(db, "repr(True)") == "True"


def test_repr_int(db, vsql_data):
	assert expr(db, "repr(-42)") == "-42"


def test_repr_number_1(db, vsql_data):
	assert expr(db, "repr(42.0)") == "42.0"


def test_repr_number_2(db, vsql_data):
	assert expr(db, "repr(-42.5)") == "-42.5"


def test_repr_str(db, vsql_data):
	assert expr(db, "repr('foo\"bar')") == "'foo\"bar'"


def test_repr_date(db, vsql_data):
	assert expr(db, "repr(@(2000-02-29))") == "@(2000-02-29)"


def test_repr_datetime(db, vsql_data):
	assert expr(db, "repr(@(2000-02-29T12:34:56))") == "@(2000-02-29T12:34:56)"


def test_repr_datedelta_1(db, vsql_data):
	assert expr(db, "repr(days(1))") == "timedelta(1)"


def test_repr_datedelta_2(db, vsql_data):
	assert expr(db, "repr(days(42))") == "timedelta(42)"


def test_repr_datetimedelta_1(db, vsql_data):
	# FIXME: Oracle doesn't have enough precision for seconds
	assert expr(db, "repr(seconds(42))") == "timedelta(0, 42)"


def test_repr_datetimedelta_2(db, vsql_data):
	assert expr(db, "repr(minutes(42))") == "timedelta(0, 2520)"


def test_repr_datetimedelta_3(db, vsql_data):
	assert expr(db, "repr(hours(17) + minutes(23))") == "timedelta(0, 62580)"


def test_repr_datetimedelta_4(db, vsql_data):
	assert expr(db, "repr(hours(42) + seconds(0))") == "timedelta(1, 64800)"


def test_repr_datetimedelta_5(db, vsql_data):
	assert expr(db, "repr(days(42) + seconds(0))") == "timedelta(42)"


def test_repr_datetimedelta_6(db, vsql_data):
	assert expr(db, "repr(days(42) + hours(17) + minutes(23))") == "timedelta(42, 62580)"


def test_repr_monthdelta(db, vsql_data):
	assert expr(db, "repr(monthdelta(42))") == "monthdelta(42)"


def test_repr_color_1(db, vsql_data):
	assert expr(db, "repr(#000)") == "#000"


def test_repr_color_2(db, vsql_data):
	assert expr(db, "repr(#369c)") == "#369c"


def test_repr_color_3(db, vsql_data):
	assert expr(db, "repr(#123456)") == "#123456"


def test_repr_color_4(db, vsql_data):
	assert expr(db, "repr(#12345678)") == "#12345678"


def test_repr_geo_without_info(db, vsql_data):
	assert expr(db, "repr(geo(49.95, 11.59))") == "<geo lat=49.95 long=11.59 info=None>"


def test_repr_geo_with_info(db, vsql_data):
	assert expr(db, "repr(geo(49.95, 11.59, 'Here'))") == "<geo lat=49.95 long=11.59 info='Here'>"


def test_repr_intlist(db, vsql_data):
	assert expr(db, "repr([1, 2, 3, None])") == "[1, 2, 3, None]"


def test_repr_numberlist(db, vsql_data):
	assert expr(db, "repr([1.2, 3.4, 5.6, None])") == "[1.2, 3.4, 5.6, None]"


def test_repr_strlist(db, vsql_data):
	assert expr(db, "repr(['foo', 'bar', None])") == "['foo', 'bar', None]"


def test_repr_datelist(db, vsql_data):
	assert expr(db, "repr([@(2000-02-29), None])") == "[@(2000-02-29), None]"


def test_repr_datetimelist(db, vsql_data):
	assert expr(db, "repr([@(2000-02-29T12:34:56), None])") == "[@(2000-02-29T12:34:56), None]"


# For the set test only include one non-``None`` value,
# as the order of the other elements is undefined

def test_repr_intset(db, vsql_data):
	assert expr(db, "repr({1, None})") == "{1, None}"


def test_repr_numberset(db, vsql_data):
	assert expr(db, "repr({1.2, None})") == "{1.2, None}"


def test_repr_strset(db, vsql_data):
	assert expr(db, "repr({'foo', None})") == "{'foo', None}"


def test_repr_dateset(db, vsql_data):
	assert expr(db, "repr({@(2000-02-29), None})") == "{@(2000-02-29), None}"


def test_repr_datetimeset(db, vsql_data):
	assert expr(db, "repr({@(2000-02-29T12:34:56), None})") == "{@(2000-02-29T12:34:56), None}"


def test_date_int(db, vsql_data):
	assert expr(db, "date(2000, 2, 29)") == datetime.datetime(2000, 2, 29)


def test_date_datetime(db, vsql_data):
	assert expr(db, "date(@(2000-02-29T12:34:56))") == datetime.datetime(2000, 2, 29)


def test_datetime_int3(db, vsql_data):
	assert expr(db, "datetime(2000, 2, 29)") == datetime.datetime(2000, 2, 29)


def test_datetime_int4(db, vsql_data):
	assert expr(db, "datetime(2000, 2, 29, 12)") == datetime.datetime(2000, 2, 29, 12)


def test_datetime_int5(db, vsql_data):
	assert expr(db, "datetime(2000, 2, 29, 12, 34)") == datetime.datetime(2000, 2, 29, 12, 34)


def test_datetime_int6(db, vsql_data):
	assert expr(db, "datetime(2000, 2, 29, 12, 34, 56)") == datetime.datetime(2000, 2, 29, 12, 34, 56)


def test_datetime_date(db, vsql_data):
	assert expr(db, "datetime(@(2000-02-29))") == datetime.datetime(2000, 2, 29)


def test_datetime_date_int1(db, vsql_data):
	assert expr(db, "datetime(@(2000-02-29), 12)") == datetime.datetime(2000, 2, 29, 12)


def test_datetime_date_int2(db, vsql_data):
	assert expr(db, "datetime(@(2000-02-29), 12, 34)") == datetime.datetime(2000, 2, 29, 12, 34)


def test_datetime_date_int3(db, vsql_data):
	assert expr(db, "datetime(@(2000-02-29), 12, 34, 56)") == datetime.datetime(2000, 2, 29, 12, 34, 56)


def test_len_str1(db, vsql_data):
	assert expr(db, "len('')") == 0


def test_len_str2(db, vsql_data):
	assert expr(db, "len('gurk')") == 4


def test_len_str3(db, vsql_data):
	assert expr(db, "len('\\t\\n')") == 2


def test_len_intlist(db, vsql_data):
	assert expr(db, "len([1, 2, 3])") == 3


def test_len_numberlist(db, vsql_data):
	assert expr(db, "len([1.2, 3.4, 5.6])") == 3


def test_len_strlist(db, vsql_data):
	assert expr(db, "len(['foo', 'bar', 'baz'])") == 3


def test_len_datelist(db, vsql_data):
	assert expr(db, "len([@(2000-02-29), @(2000-02-29), @(2000-03-01)])") == 3


def test_len_datetimelist(db, vsql_data):
	assert expr(db, "len([@(2000-02-29T12:34:56), @(2000-02-29T12:34:56), @(2000-03-01T12:34:56)])") == 3


def test_len_intset(db, vsql_data):
	assert expr(db, "len({1, 1, 2, 2, 3, 3, None, None})") == 4


def test_len_numberset(db, vsql_data):
	assert expr(db, "len({1.2, 3.4, 5.6, None, 1.2, 3.4, 5.6, None})") == 4


def test_len_strset(db, vsql_data):
	assert expr(db, "len({'foo', 'bar', 'baz', None, 'foo', 'bar', 'baz'})") == 4


def test_len_dateset(db, vsql_data):
	assert expr(db, "len({@(2000-02-29), @(2000-02-29), @(2000-03-21), None})") == 3


def test_len_datetimeset(db, vsql_data):
	assert expr(db, "len({@(2000-02-29T12:34:56), None, @(2000-02-29T12:34:56), None, @(2000-02-29T11:22:33)})") == 3


def test_timedelta(db, vsql_data):
	assert expr(db, "timedelta()") == False


def test_timedelta_int1(db, vsql_data):
	assert expr(db, "timedelta(42)") == 42


def test_timedelta_int2(db, vsql_data):
	assert expr(db, "timedelta(42, 12)") == 42 + 12/24/60/60


def test_monthdelta(db, vsql_data):
	assert expr(db, "monthdelta()") == 0


def test_monthdelta_int(db, vsql_data):
	assert expr(db, "monthdelta(42)") == 42


def test_years(db, vsql_data):
	assert expr(db, "years(25)") == 25 * 12


def test_months(db, vsql_data):
	assert expr(db, "months(3)") == 3


def test_weeks(db, vsql_data):
	assert expr(db, "weeks(3)") == 3 * 7


def test_days(db, vsql_data):
	assert expr(db, "days(12)") == 12


def test_hours(db, vsql_data):
	assert expr(db, "hours(8)") == 8/24


def test_minutes(db, vsql_data):
	assert expr(db, "minutes(45)") == 45/24/60


def test_seconds(db, vsql_data):
	assert expr(db, "seconds(60)") == pytest.approx(60/24/60/60)


def test_md5(db, vsql_data):
	assert expr(db, "md5('gurk')") == "4b5b6a3fa4af2541daa569277c7ff4c5"


def test_random(db, vsql_data):
	assert 1.0 <= expr(db, "random() + 1") <= 2.0


def test_randrange(db, vsql_data):
	assert 0 <= expr(db, "randrange(1, 10)") < 10


def test_seq(db, vsql_data):
	assert expr(db, "seq()")


def test_rgb1(db, vsql_data):
	assert expr(db, "rgb(0.2, 0.4, 0.6)") == 0x336699ff


def test_rgb2(db, vsql_data):
	assert expr(db, "rgb(0.2, 0.4, 0.6, 0.8)") == 0x336699cc


def test_list_str(db, vsql_data):
	assert expr(db, "list('gurk')") == ["g", "u", "r", "k"]


def test_list_intlist(db, vsql_data):
	assert expr(db, "list([1, 2, 3])") == [1, 2, 3]


def test_list_numberlist(db, vsql_data):
	assert expr(db, "list([1.2, 3.4, 5.6])") == [1.2, 3.4, 5.6]


def test_list_strlist(db, vsql_data):
	assert expr(db, "list(['foo', 'bar', 'baz', None])") == ["foo", "bar", "baz", None]


def test_list_datelist(db, vsql_data):
	assert expr(db, "list([@(2000-02-29), @(2000-03-01), None])") == [datetime.datetime(2000, 2, 29), datetime.datetime(2000, 3, 1), None]


def test_list_datetimelist(db, vsql_data):
	assert expr(db, "list([@(2000-02-29T12:34:56), @(2000-02-29T11:22:33), None])") == [datetime.datetime(2000, 2, 29, 12, 34, 56), datetime.datetime(2000, 2, 29, 11, 22, 33), None]


def test_list_intset(db, vsql_data):
	assert expr(db, "list({1, None})") == [1, None]


def test_list_numberset(db, vsql_data):
	assert expr(db, "list({1.2, None})") == [1.2, None]


def test_list_strset(db, vsql_data):
	assert expr(db, "list({'foo', None})") == ['foo', None]


def test_list_dateset(db, vsql_data):
	assert expr(db, "list({@(2000-02-29), None})") == [datetime.datetime(2000, 2, 29), None]


def test_list_datetimeset(db, vsql_data):
	assert expr(db, "list({@(2000-02-29T12:34:56), None})") == [datetime.datetime(2000, 2, 29, 12, 34, 56), None]


def test_set_str(db, vsql_data):
	assert set(expr(db, "set('mississippi')")) == {"i", "m", "p", "s"}


def test_set_intlist(db, vsql_data):
	assert set(expr(db, "set([1, 2, 3, 2, 1, None])")) == {1, 2, 3, None}


def test_set_numberlist(db, vsql_data):
	assert set(expr(db, "set([1.2, 3.4, 5.6, 3.4, 1.2, None])")) == {1.2, 3.4, 5.6, None}


def test_set_strlist(db, vsql_data):
	assert set(expr(db, "set(['foo', 'bar', 'baz', None, 'baz', 'bar', 'foo'])")) == {"foo", "bar", "baz", None}


def test_set_datelist(db, vsql_data):
	assert set(expr(db, "set([@(2000-02-29), @(2000-03-01), None, @(2000-03-01), @(2000-02-29)])")) == {datetime.datetime(2000, 2, 29), datetime.datetime(2000, 3, 1), None}


def test_set_datetimelist(db, vsql_data):
	assert set(expr(db, "set([@(2000-02-29T12:34:56), @(2000-02-29T11:22:33), @(2000-02-29T11:22:33), None, @(2000-02-29T12:34:56)])")) == {datetime.datetime(2000, 2, 29, 12, 34, 56), datetime.datetime(2000, 2, 29, 11, 22, 33), None}


def test_set_intset(db, vsql_data):
	assert set(expr(db, "set({1, None})")) == {1, None}


def test_set_numberset(db, vsql_data):
	assert set(expr(db, "set({1.2, None})")) == {1.2, None}


def test_set_strset(db, vsql_data):
	assert set(expr(db, "set({'foo', None})")) == {'foo', None}


def test_set_dateset(db, vsql_data):
	assert set(expr(db, "set({@(2000-02-29), None})")) == {datetime.datetime(2000, 2, 29), None}


def test_set_datetimeset(db, vsql_data):
	assert set(expr(db, "set({@(2000-02-29T12:34:56), None})")) == {datetime.datetime(2000, 2, 29, 12, 34, 56), None}


def test_dist(db, vsql_data):
	assert abs(expr(db, "dist(geo(49.95, 11.59, 'Here'), geo(12.34, 56.67, 'There'))")) - 5845.77551787602 < 1e-5


def test_abs_bool(db, vsql_data):
	assert expr(db, "abs(False)") == 0


def test_abs_int(db, vsql_data):
	assert expr(db, "abs(-42)") == 42


def test_abs_number(db, vsql_data):
	assert expr(db, "abs(-42.5)") == 42.5


def test_cos_bool(db, vsql_data):
	assert expr(db, "cos(False)") == 1.0


def test_cos_int(db, vsql_data):
	assert expr(db, "cos(0)") == 1.0


def test_cos_number1(db, vsql_data):
	assert expr(db, "cos(0.0)") == 1.0


def test_cos_number2(db, vsql_data):
	assert abs(expr(db, f"cos({math.pi} / 2)")) < 1e-10


def test_cos_number3(db, vsql_data):
	assert abs(expr(db, f"cos({math.pi})") + 1) < 1e-10


def test_sin_bool(db, vsql_data):
	assert expr(db, "sin(False)") == 0.0


def test_sin_int(db, vsql_data):
	assert expr(db, "sin(0)") == 0.0


def test_sin_number1(db, vsql_data):
	assert expr(db, "sin(0.0)") == 0.0


def test_sin_number2(db, vsql_data):
	assert abs(expr(db, f"sin({math.pi} / 2)") - 1) < 1e-10


def test_sin_number3(db, vsql_data):
	assert abs(expr(db, f"sin({math.pi})")) < 1e-10


def test_tan_bool(db, vsql_data):
	assert expr(db, "tan(False)") == 0.0


def test_tan_int(db, vsql_data):
	assert expr(db, "tan(0)") == 0.0


def test_tan_number1(db, vsql_data):
	assert expr(db, "tan(0.0)") == 0.0


def test_tan_number2(db, vsql_data):
	assert abs(expr(db, f"tan(0.25 * {math.pi})") - 1) < 1e-10


def test_tan_number3(db, vsql_data):
	assert abs(expr(db, f"tan(0.75 * {math.pi})") + 1) < 1e-10


def test_sqrt_bool1(db, vsql_data):
	assert expr(db, "sqrt(False)") == 0.0


def test_sqrt_bool2(db, vsql_data):
	assert expr(db, "sqrt(True)") == 1.0


def test_sqrt_int1(db, vsql_data):
	assert expr(db, "sqrt(16)") == 4.0


def test_sqrt_int2(db, vsql_data):
	assert expr(db, "sqrt(-16)") is None


def test_sqrt_number1(db, vsql_data):
	assert expr(db, "sqrt(16.0)") == 4.0


def test_sqrt_number2(db, vsql_data):
	assert expr(db, "sqrt(-16.0)") is None



"""
Tests for the vSQL addition operator ``+``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

d1_v = "@(2000-02-29)"
d2_v = "@(2000-03-01)"

dt1_v = "@(2000-02-29T12:34:56)"
dt2_v = "@(2000-03-01T12:34:56)"

d1 = datetime.datetime(2000, 2, 29)
d2 = datetime.datetime(2000, 3, 1)

dt1 = datetime.datetime(2000, 2, 29, 12, 34, 56)
dt2 = datetime.datetime(2000, 3, 1, 12, 34, 56)


def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_bool + True", where="r.identifier == 'none'") is None


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool + True", where="r.identifier == 'bool_false'") == 1


def test_bool_bool3(db, vsql_data):
	assert expr(db, "r.v_bool + True", where="r.identifier == 'bool_true'") == 2


def test_bool_int(db, vsql_data):
	assert expr(db, "r.v_bool + 1", where="r.identifier == 'bool_true'") == 2


def test_bool_number(db, vsql_data):
	assert expr(db, "r.v_bool + 1.5", where="r.identifier == 'bool_true'") == 2.5


def test_int_bool(db, vsql_data):
	assert expr(db, "1 + r.v_bool", where="r.identifier == 'bool_true'") == 2


def test_int_int(db, vsql_data):
	assert expr(db, "1 + r.v_int", where="r.identifier == 'int'") == 1777


def test_int_number(db, vsql_data):
	assert expr(db, "1 + r.v_number", where="r.identifier == 'number'") == 43.5


def test_str_str1(db, vsql_data):
	assert expr(db, "'gurk' + r.v_str", where="r.identifier == 'none'") == "gurk"


def test_str_str2(db, vsql_data):
	assert expr(db, "'gurk' + r.v_str", where="r.identifier == 'str'") == "gurkgurk"


def test_str_clob1(db, vsql_data):
	assert expr(db, "'hurz' + r.v_clob", where="r.identifier == 'none'") == "hurz"


def test_str_clob2(db, vsql_data):
	assert expr(db, "'hurz' + r.v_clob", where="r.identifier == 'clob'") == "hurz" + "gurk" * 100000


def test_clob_str1(db, vsql_data):
	assert expr(db, "r.v_clob + 'hurz'", where="r.identifier == 'none'") == "hurz"


def test_clob_str2(db, vsql_data):
	assert expr(db, "r.v_clob + 'hurz'", where="r.identifier == 'clob'") == "gurk" * 100000 + "hurz"


def test_intlist_intlist(db, vsql_data):
	assert expr(db, "[1, 2] + [3, 4]") == [1, 2, 3, 4]


def test_intlist_numberlist(db, vsql_data):
	assert expr(db, "[1, 2] + [3.5, 4.5]") == [1.0, 2.0, 3.5, 4.5]


def test_numberlist_intlist(db, vsql_data):
	assert expr(db, "[1.5, 2.5] + [3, 4]") == [1.5, 2.5, 3.0, 4.0]


def test_numberlist_numberlist(db, vsql_data):
	assert expr(db, "[1.5, 2.5] + [3.5, 4.5]") == [1.5, 2.5, 3.5, 4.5]


def test_strlist_strlist(db, vsql_data):
	assert expr(db, "['gurk', 'hurz'] + ['hinz', 'kunz']") == ['gurk', 'hurz', 'hinz', 'kunz']


def test_datelist_datelist(db, vsql_data):
	assert expr(db, "[@(2000-02-29), @(2000-03-01)] + [@(2000-03-02), @(2000-03-03)]") == [datetime.datetime(2000, 2, 29), datetime.datetime(2000, 3, 1), datetime.datetime(2000, 3, 2), datetime.datetime(2000, 3, 3)]


def test_datetimelist_datetimelist(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)] + [@(2000-03-02T12:34:56), @(2000-03-03T12:34:56)]") == [datetime.datetime(2000, 2, 29, 12, 34, 56), datetime.datetime(2000, 3, 1, 12, 34, 56), datetime.datetime(2000, 3, 2, 12, 34, 56), datetime.datetime(2000, 3, 3, 12, 34, 56)]


def test_date_datedelta(db, vsql_data):
	assert expr(db, "r.v_date + days(1)", where="r.identifier == 'date'") == datetime.datetime(2000, 3, 1)


def test_date_monthdelta(db, vsql_data):
	assert expr(db, "@(2000-01-31) + months(1)") == datetime.datetime(2000, 2, 29)


def test_datetime_datedelta(db, vsql_data):
	assert expr(db, "r.v_datetime + days(1)", where="r.identifier == 'datetime'") == datetime.datetime(2000, 3, 1, 12, 34, 56)


def test_datetime_datetimedelta(db, vsql_data):
	assert expr(db, "r.v_datetime + timedelta(1, 1)", where="r.identifier == 'datetime'") == datetime.datetime(2000, 3, 1, 12, 34, 57)


def test_datetime_monthdelta(db, vsql_data):
	assert expr(db, "@(2000-01-31T12:34:56) + months(1)") == datetime.datetime(2000, 2, 29, 12, 34, 56)


def test_monthdelta_date(db, vsql_data):
	assert expr(db, "months(1) + @(2000-01-31)") == datetime.datetime(2000, 2, 29)


def test_monthdelta_datetime(db, vsql_data):
	assert expr(db, "months(1) + @(2000-01-31T12:34:56)") == datetime.datetime(2000, 2, 29,12, 34, 56)


def test_datedelta_datedelta(db, vsql_data):
	assert expr(db, "r.v_datedelta + days(12)", where="r.identifier == 'datedelta'") == 24


def test_datedelta_datetimedelta(db, vsql_data):
	assert expr(db, "r.v_datedelta + timedelta(1, 1)", where="r.identifier == 'datedelta'") == 13.0 + 1/24/60/60


def test_datetimedelta_datedelta(db, vsql_data):
	assert expr(db, "r.v_datetimedelta + days(12)", where="r.identifier == 'datetimedelta'") == 13 + 12/24 + 34/24/60 + 56/24/60/60


def test_datetimedelta_datetimedelta(db, vsql_data):
	assert expr(db, "r.v_datetimedelta + timedelta(2, (12 * 60 + 34) * 60 + 56)", where="r.identifier == 'datetimedelta'") == 3 + 2 * (12/24 + 34/24/60 + 56/24/60/60)


def test_monthdelta_monthdelta(db, vsql_data):
	assert expr(db, "r.v_monthdelta + months(9)", where="r.identifier == 'monthdelta'") == 12


def test_nulllist_nulllist1(db, vsql_data):
	assert expr(db, "[] + []") == 0


def test_nulllist_nulllist2(db, vsql_data):
	assert expr(db, "[None, None] + [None]") == 3


def test_nulllist_intlist1(db, vsql_data):
	assert expr(db, "[] + [1, None, 2]") == [1, None, 2]


def test_nulllist_intlist2(db, vsql_data):
	assert expr(db, "[None, None] + [1, None, 2]") == [None, None, 1, None, 2]


def test_nulllist_numberlist1(db, vsql_data):
	assert expr(db, "[] + [1.1, None, 2.2]") == [1.1, None, 2.2]


def test_nulllist_numberlist2(db, vsql_data):
	assert expr(db, "[None, None] + [1.1, None, 2.2]") == [None, None, 1.1, None, 2.2]


def test_nulllist_strlist1(db, vsql_data):
	assert expr(db, "[] + ['gurk', None, 'hurz']") == ['gurk', None, 'hurz']


def test_nulllist_strlist2(db, vsql_data):
	assert expr(db, "[None, None] + ['gurk', None, 'hurz']") == [None, None, 'gurk', None, 'hurz']


def test_nulllist_datelist1(db, vsql_data):
	assert expr(db, f"[] + [{d1_v}, None, {d2_v}]") == [d1, None, d2]


def test_nulllist_datelist2(db, vsql_data):
	assert expr(db, f"[None, None] + [{d1_v}, None, {d2_v}]") == [None, None, d1, None, d2]


def test_nulllist_datetimelist1(db, vsql_data):
	assert expr(db, f"[] + [{dt1_v}, None, {dt2_v}]") == [dt1, None, dt2]


def test_nulllist_datetimelist2(db, vsql_data):
	assert expr(db, f"[None, None] + [{dt1_v}, None, {dt2_v}]") == [None, None, dt1, None, dt2]


def test_intlist_nulllist1(db, vsql_data):
	assert expr(db, "[1, None, 2] + []") == [1, None, 2]


def test_intlist_nulllist2(db, vsql_data):
	assert expr(db, "[1, None, 2] + [None, None]") == [1, None, 2, None, None]


def test_numberlist_nulllist1(db, vsql_data):
	assert expr(db, "[1.1, None, 2.2] + []") == [1.1, None, 2.2]


def test_numberlist_nulllist2(db, vsql_data):
	assert expr(db, "[1.1, None, 2.2] + [None, None]") == [1.1, None, 2.2, None, None]


def test_strlist_nulllist1(db, vsql_data):
	assert expr(db, "['gurk', None, 'hurz'] + []") == ['gurk', None, 'hurz']


def test_strlist_nulllist2(db, vsql_data):
	assert expr(db, "['gurk', None, 'hurz'] + [None, None]") == ['gurk', None, 'hurz', None, None]


def test_datelist_nulllist1(db, vsql_data):
	assert expr(db, f"[{d1_v}, None, {d2_v}] + []") == [d1, None, d2]


def test_datelist_nulllist2(db, vsql_data):
	assert expr(db, f"[{d1_v}, None, {d2_v}] + [None, None]") == [d1, None, d2, None, None]


def test_datetimelist_nulllist1(db, vsql_data):
	assert expr(db, f"[{dt1_v}, None, {dt2_v}] + []") == [dt1, None, dt2]


def test_datetimelist_nulllist2(db, vsql_data):
	assert expr(db, f"[{dt1_v}, None, {dt2_v}] + [None, None]") == [dt1, None, dt2, None, None]



"""
Tests for the vSQL multiplication operator ``*``.

To run the tests, :mod:`pytest` is required.
"""

import datetime
from conftest import *


###
### Tests
###

def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_int * True", where="r.identifier == 'none'") is None


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool * True", where="r.identifier == 'bool_false'") == 0


def test_bool_bool3(db, vsql_data):
	assert expr(db, "r.v_bool * True", where="r.identifier == 'bool_true'") == 1


def test_bool_int(db, vsql_data):
	assert expr(db, "r.v_bool * 1", where="r.identifier == 'bool_true'") == 1


def test_bool_number(db, vsql_data):
	assert expr(db, "r.v_bool * 1.5", where="r.identifier == 'bool_true'") == 1.5


def test_int_bool(db, vsql_data):
	assert expr(db, "2 * r.v_bool", where="r.identifier == 'bool_true'") == 2


def test_int_int(db, vsql_data):
	assert expr(db, "2 * r.v_int", where="r.identifier == 'int'") == 3552


def test_int_number(db, vsql_data):
	assert expr(db, "2 * r.v_number", where="r.identifier == 'number'") == 85.0


def test_number_bool(db, vsql_data):
	assert expr(db, "r.v_number * True", where="r.identifier == 'number'") == 42.5


def test_number_int(db, vsql_data):
	assert expr(db, "r.v_number * 2", where="r.identifier == 'number'") == 85.0


def test_number_number(db, vsql_data):
	assert expr(db, "r.v_number * 1.5", where="r.identifier == 'number'") == 63.75


def test_bool_str1(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_str", where="r.identifier == 'none'") is None


def test_bool_str2(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_str", where="r.identifier == 'bool_false'") is None


def test_bool_str3(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_str", where="r.identifier == 'bool_false'") is None


def test_bool_str4(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_str", where="r.identifier == 'str'") is None


def test_bool_str5(db, vsql_data):
	assert expr(db, "False * r.v_str", where="r.identifier == 'str'") is None


def test_bool_str6(db, vsql_data):
	assert expr(db, "True * r.v_str", where="r.identifier == 'str'") == "gurk"


def test_int_str1(db, vsql_data):
	assert expr(db, "r.v_int * r.v_str", where="r.identifier == 'none'") is None


def test_int_str2(db, vsql_data):
	assert expr(db, "2 * r.v_str", where="r.identifier == 'none'") is None


def test_int_str3(db, vsql_data):
	assert expr(db, "r.v_int * r.v_str", where="r.identifier == 'str'") is None


def test_int_str4(db, vsql_data):
	assert expr(db, "2 * r.v_str", where="r.identifier == 'str'") == "gurkgurk"


def test_bool_clob1(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_clob", where="r.identifier == 'none'") is None


def test_bool_clob2(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_clob", where="r.identifier == 'bool_false'") is None


def test_bool_clob3(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_clob", where="r.identifier == 'bool_false'") is None


def test_bool_clob4(db, vsql_data):
	assert expr(db, "r.v_bool * r.v_clob", where="r.identifier == 'clob'") is None


def test_bool_clob5(db, vsql_data):
	result = expr(db, "False * r.v_clob", where="r.identifier == 'clob'")
	assert result is None or result == ""


def test_bool_clob6(db, vsql_data):
	assert expr(db, "True * r.v_clob", where="r.identifier == 'clob'") == "gurk" * 100000


def test_int_clob1(db, vsql_data):
	assert expr(db, "r.v_int * r.v_clob", where="r.identifier == 'none'") is None


def test_int_clob2(db, vsql_data):
	assert expr(db, "2 * r.v_clob", where="r.identifier == 'none'") is None


def test_int_clob3(db, vsql_data):
	assert expr(db, "r.v_int * r.v_clob", where="r.identifier == 'clob'") is None


def test_int_clob4(db, vsql_data):
	assert expr(db, "2 * r.v_clob", where="r.identifier == 'clob'") == "gurk" * 200000


def test_bool_datedelta1(db, vsql_data):
	assert expr(db, "r.v_bool * days(3)", where="r.identifier == 'none'") is None


def test_bool_datedelta2(db, vsql_data):
	assert expr(db, "True * r.v_datedelta", where="r.identifier == 'none'") is None


def test_bool_datedelta3(db, vsql_data):
	assert expr(db, "True * r.v_datedelta", where="r.identifier == 'datedelta'") == 12


def test_int_datedelta1(db, vsql_data):
	assert expr(db, "r.v_int * days(3)", where="r.identifier == 'none'") is None


def test_int_datedelta2(db, vsql_data):
	assert expr(db, "2 * r.v_datedelta", where="r.identifier == 'none'") is None


def test_int_datedelta3(db, vsql_data):
	assert expr(db, "2 * r.v_datedelta", where="r.identifier == 'datedelta'") == 24


def test_bool_datetimedelta1(db, vsql_data):
	assert expr(db, "r.v_bool * minutes(3)", where="r.identifier == 'none'") is None


def test_bool_datetimedelta2(db, vsql_data):
	assert expr(db, "True * r.v_datetimedelta", where="r.identifier == 'none'") is None


def test_bool_datetimedelta3(db, vsql_data):
	assert expr(db, "True * r.v_datetimedelta", where="r.identifier == 'datetimedelta'") == 1 + 12/24 + 34/24/60 + 56/24/60/60


def test_int_datetimedelta1(db, vsql_data):
	assert expr(db, "r.v_int * minutes(3)", where="r.identifier == 'none'") is None


def test_int_datetimedelta2(db, vsql_data):
	assert expr(db, "2 * r.v_datetimedelta", where="r.identifier == 'none'") is None


def test_int_datetimedelta3(db, vsql_data):
	assert expr(db, "2 * r.v_datetimedelta", where="r.identifier == 'datetimedelta'") == 2 * (1 + 12/24 + 34/24/60 + 56/24/60/60)


def test_bool_monthdelta1(db, vsql_data):
	assert expr(db, "r.v_bool * months(3)", where="r.identifier == 'none'") is None


def test_bool_monthdelta2(db, vsql_data):
	assert expr(db, "True * r.v_monthdelta", where="r.identifier == 'none'") is None


def test_bool_monthdelta3(db, vsql_data):
	assert expr(db, "True * r.v_monthdelta", where="r.identifier == 'monthdelta'") == 3


def test_int_monthdelta1(db, vsql_data):
	assert expr(db, "r.v_int * months(3)", where="r.identifier == 'none'") is None


def test_int_monthdelta2(db, vsql_data):
	assert expr(db, "2 * r.v_monthdelta", where="r.identifier == 'none'") is None


def test_int_monthdelta3(db, vsql_data):
	assert expr(db, "2 * r.v_monthdelta", where="r.identifier == 'monthdelta'") == 6


def test_number_datetimedelta3(db, vsql_data):
	assert expr(db, "2.5 * r.v_datetimedelta", where="r.identifier == 'datetimedelta'") == 2.5 * (1 + 12/24 + 34/24/60 + 56/24/60/60)


def test_str_bool1(db, vsql_data):
	assert expr(db, "r.v_str * r.v_bool", where="r.identifier == 'none'") is None


def test_str_bool2(db, vsql_data):
	assert expr(db, "r.v_str * False", where="r.identifier == 'none'") is None


def test_str_bool3(db, vsql_data):
	assert expr(db, "r.v_str * True", where="r.identifier == 'none'") is None


def test_str_bool4(db, vsql_data):
	assert expr(db, "r.v_str * r.v_bool", where="r.identifier == 'str'") is None


def test_str_bool5(db, vsql_data):
	assert expr(db, "r.v_str * False", where="r.identifier == 'str'") is None


def test_str_bool6(db, vsql_data):
	assert expr(db, "r.v_str * True", where="r.identifier == 'str'") == "gurk"


def test_str_int1(db, vsql_data):
	assert expr(db, "r.v_str * r.v_int", where="r.identifier == 'none'") is None


def test_str_int2(db, vsql_data):
	assert expr(db, "r.v_str * 2", where="r.identifier == 'none'") is None


def test_str_int3(db, vsql_data):
	assert expr(db, "r.v_str * r.v_int", where="r.identifier == 'str'") is None


def test_bool_intlist1(db, vsql_data):
	assert expr(db, "r.v_bool * [1, 2, 3]", where="r.identifier == 'none'") is None


def test_bool_intlist2(db, vsql_data):
	assert expr(db, "r.v_bool * [1, 2, 3]", where="r.identifier == 'bool_false'") == []


def test_bool_intlist3(db, vsql_data):
	assert expr(db, "r.v_bool * [1, 2, 3]", where="r.identifier == 'bool_true'") == [1, 2, 3]


def test_int_intlist1(db, vsql_data):
	assert expr(db, "r.v_int * [1, 2, 3]", where="r.identifier == 'none'") is None


def test_int_intlist2(db, vsql_data):
	assert expr(db, "2 * [1, 2, 3]") == [1, 2, 3, 1, 2, 3]


def test_bool_nulllist1(db, vsql_data):
	assert expr(db, "False * []") == 0


def test_bool_nulllist2(db, vsql_data):
	assert expr(db, "True * []") == 0


def test_bool_nulllist3(db, vsql_data):
	assert expr(db, "False * [None, None]") == 0


def test_bool_nulllist4(db, vsql_data):
	assert expr(db, "True * [None, None]") == 2


def test_int_nulllist1(db, vsql_data):
	assert expr(db, "0 * []") == 0


def test_int_nulllist2(db, vsql_data):
	assert expr(db, "2 * []") == 0


def test_int_nulllist3(db, vsql_data):
	assert expr(db, "0 * [None, None]") == 0


def test_int_nulllist4(db, vsql_data):
	assert expr(db, "2 * [None, None]") == 4


def test_nulllist1_bool(db, vsql_data):
	assert expr(db, "[] * False") == 0


def test_nulllist2_bool(db, vsql_data):
	assert expr(db, "[] * True") == 0


def test_nulllist3_bool(db, vsql_data):
	assert expr(db, "[None, None] * False") == 0


def test_nulllist4_bool(db, vsql_data):
	assert expr(db, "[None, None] * True") == 2


def test_nulllist1_int(db, vsql_data):
	assert expr(db, "[] * 0") == 0


def test_nulllist2_int(db, vsql_data):
	assert expr(db, "[] * 2") == 0


def test_nulllist3_int(db, vsql_data):
	assert expr(db, "[None, None] * 0") == 0


def test_nulllist4_int(db, vsql_data):
	assert expr(db, "[None, None] * 2") == 4


"""
Tests for the vSQL binary identity test operator ``is``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool1(db, vsql_data):
	assert expr(db, "r.v_bool is None", where="r.identifier == 'none'") == 1

def test_bool2(db, vsql_data):
	assert expr(db, "r.v_bool is None", where="r.identifier == 'bool_false'") == 0

def test_bool3(db, vsql_data):
	assert expr(db, "r.v_bool is None", where="r.identifier == 'bool_true'") == 0

def test_bool4(db, vsql_data):
	assert expr(db, "None is r.v_bool", where="r.identifier == 'none'") == 1

def test_bool5(db, vsql_data):
	assert expr(db, "None is r.v_bool", where="r.identifier == 'bool_false'") == 0

def test_bool6(db, vsql_data):
	assert expr(db, "None is r.v_bool", where="r.identifier == 'bool_true'") == 0

def test_int1(db, vsql_data):
	assert expr(db, "r.v_int is None", where="r.identifier == 'none'") == 1

def test_int2(db, vsql_data):
	assert expr(db, "r.v_int is None", where="r.identifier == 'int'") == 0

def test_int3(db, vsql_data):
	assert expr(db, "None is r.v_int", where="r.identifier == 'none'") == 1

def test_int4(db, vsql_data):
	assert expr(db, "None is r.v_int", where="r.identifier == 'int'") == 0

def test_number1(db, vsql_data):
	assert expr(db, "r.v_number is None", where="r.identifier == 'none'") == 1

def test_number2(db, vsql_data):
	assert expr(db, "r.v_number is None", where="r.identifier == 'number'") == 0

def test_number3(db, vsql_data):
	assert expr(db, "None is r.v_number", where="r.identifier == 'none'") == 1

def test_number4(db, vsql_data):
	assert expr(db, "None is r.v_number", where="r.identifier == 'number'") == 0

def test_str1(db, vsql_data):
	assert expr(db, "r.v_str is None", where="r.identifier == 'none'") == 1

def test_str2(db, vsql_data):
	assert expr(db, "r.v_str is None", where="r.identifier == 'str'") == 0

def test_str3(db, vsql_data):
	assert expr(db, "None is r.v_str", where="r.identifier == 'none'") == 1

def test_str4(db, vsql_data):
	assert expr(db, "None is r.v_str", where="r.identifier == 'str'") == 0

def test_color1(db, vsql_data):
	assert expr(db, "r.v_color is None", where="r.identifier == 'none'") == 1

def test_color2(db, vsql_data):
	assert expr(db, "r.v_color is None", where="r.identifier == 'color'") == 0

def test_color3(db, vsql_data):
	assert expr(db, "None is r.v_color", where="r.identifier == 'none'") == 1

def test_color4(db, vsql_data):
	assert expr(db, "None is r.v_color", where="r.identifier == 'color'") == 0

def test_date1(db, vsql_data):
	assert expr(db, "r.v_date is None", where="r.identifier == 'none'") == 1

def test_date2(db, vsql_data):
	assert expr(db, "r.v_date is None", where="r.identifier == 'date'") == 0

def test_date3(db, vsql_data):
	assert expr(db, "None is r.v_date", where="r.identifier == 'none'") == 1

def test_date4(db, vsql_data):
	assert expr(db, "None is r.v_date", where="r.identifier == 'date'") == 0

def test_datetime1(db, vsql_data):
	assert expr(db, "r.v_datetime is None", where="r.identifier == 'none'") == 1

def test_datetime2(db, vsql_data):
	assert expr(db, "r.v_datetime is None", where="r.identifier == 'datetime'") == 0

def test_datetime3(db, vsql_data):
	assert expr(db, "None is r.v_datetime", where="r.identifier == 'none'") == 1

def test_datetime4(db, vsql_data):
	assert expr(db, "None is r.v_datetime", where="r.identifier == 'datetime'") == 0

def test_datedelta1(db, vsql_data):
	assert expr(db, "r.v_datedelta is None", where="r.identifier == 'none'") == 1

def test_datedelta2(db, vsql_data):
	assert expr(db, "r.v_datedelta is None", where="r.identifier == 'datedelta'") == 0

def test_datedelta3(db, vsql_data):
	assert expr(db, "None is r.v_datedelta", where="r.identifier == 'none'") == 1

def test_datedelta4(db, vsql_data):
	assert expr(db, "None is r.v_datedelta", where="r.identifier == 'datedelta'") == 0

def test_datetimedelta1(db, vsql_data):
	assert expr(db, "r.v_datetimedelta is None", where="r.identifier == 'none'") == 1

def test_datetimedelta2(db, vsql_data):
	assert expr(db, "r.v_datetimedelta is None", where="r.identifier == 'datetimedelta'") == 0

def test_datetimedelta3(db, vsql_data):
	assert expr(db, "None is r.v_datetimedelta", where="r.identifier == 'none'") == 1

def test_datetimedelta4(db, vsql_data):
	assert expr(db, "None is r.v_datetimedelta", where="r.identifier == 'datetimedelta'") == 0

def test_monthdelta1(db, vsql_data):
	assert expr(db, "r.v_monthdelta is None", where="r.identifier == 'none'") == 1

def test_monthdelta2(db, vsql_data):
	assert expr(db, "r.v_monthdelta is None", where="r.identifier == 'monthdelta'") == 0

def test_monthdelta3(db, vsql_data):
	assert expr(db, "None is r.v_monthdelta", where="r.identifier == 'none'") == 1

def test_monthdelta4(db, vsql_data):
	assert expr(db, "None is r.v_monthdelta", where="r.identifier == 'monthdelta'") == 0

def test_geo1(db, vsql_data):
	assert expr(db, "geo(49, 11, 'Here') is None") == 0

def test_geo2(db, vsql_data):
	assert expr(db, "None is geo(49, 11, 'Here')") == 0

def test_intlist1(db, vsql_data):
	assert expr(db, "[1, 2, 3] is None") == 0

def test_intlist2(db, vsql_data):
	assert expr(db, "None is [1, 2, 3]") == 0

def test_numberlist1(db, vsql_data):
	assert expr(db, "[1.1, 2.2, 3.3] is None") == 0

def test_numberlist2(db, vsql_data):
	assert expr(db, "None is [1.1, 2.2, 3.3]") == 0

def test_strlist1(db, vsql_data):
	assert expr(db, "['gurk', 'hurz'] is None") == 0

def test_strlist2(db, vsql_data):
	assert expr(db, "None is ['gurk', 'hurz']") == 0

def test_datelist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29), @(2000-03-01)] is None") == 0

def test_datelist2(db, vsql_data):
	assert expr(db, "None is [@(2000-02-29), @(2000-03-01)]") == 0

def test_datetimelist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)] is None") == 0

def test_datetimelist2(db, vsql_data):
	assert expr(db, "None is [@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)]") == 0



"""
Tests for the vSQL inequality comparision operator ``!=``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import expr


###
### Tests
###

def test_bool_none_false(db, vsql_data):
	assert not expr(db, "r.v_bool != None", where="r.identifier == 'none'")


def test_bool_none_true(db, vsql_data):
	assert expr(db, "r.v_bool != None", where="r.identifier == 'bool_true'")


def test_int_none_false(db, vsql_data):
	assert not expr(db, "r.v_int != None", where="r.identifier == 'none'")


def test_int_none_true(db, vsql_data):
	assert expr(db, "r.v_int != None", where="r.identifier == 'int'")


def test_number_none_false(db, vsql_data):
	assert not expr(db, "r.v_number != None", where="r.identifier == 'none'")


def test_number_none_true(db, vsql_data):
	assert expr(db, "r.v_number != None", where="r.identifier == 'number'")


def test_str_none_false(db, vsql_data):
	assert not expr(db, "r.v_str != None", where="r.identifier == 'none'")


def test_str_none_true(db, vsql_data):
	assert expr(db, "r.v_str != None", where="r.identifier == 'str'")


def test_date_none_false(db, vsql_data):
	assert not expr(db, "r.v_date != None", where="r.identifier == 'none'")


def test_date_none_true(db, vsql_data):
	assert expr(db, "r.v_date != None", where="r.identifier == 'date'")


def test_datetime_none_false(db, vsql_data):
	assert not expr(db, "r.v_datetime != None", where="r.identifier == 'none'")


def test_datetime_none_true(db, vsql_data):
	assert expr(db, "r.v_datetime != None", where="r.identifier == 'datetime'")


def test_color_none_false(db, vsql_data):
	assert not expr(db, "r.v_color != None", where="r.identifier == 'none'")


def test_color_none_true(db, vsql_data):
	assert expr(db, "r.v_color != None", where="r.identifier == 'color'")


def test_datedelta_none_false(db, vsql_data):
	assert not expr(db, "r.v_datedelta != None", where="r.identifier == 'none'")


def test_datedelta_none_true(db, vsql_data):
	assert expr(db, "r.v_datedelta != None", where="r.identifier == 'datedelta'")


def test_datetimedelta_none_false(db, vsql_data):
	assert not expr(db, "r.v_datetimedelta != None", where="r.identifier == 'none'")


def test_datetimedelta_none_true(db, vsql_data):
	assert expr(db, "r.v_datetimedelta != None", where="r.identifier == 'datetimedelta'")


def test_monthdelta_none_false(db, vsql_data):
	assert not expr(db, "r.v_monthdelta != None", where="r.identifier == 'none'")


def test_monthdelta_none_true(db, vsql_data):
	assert expr(db, "r.v_monthdelta != None", where="r.identifier == 'monthdelta'")


def test_bool_bool_false(db, vsql_data):
	assert not expr(db, "r.v_bool != False", where="r.identifier == 'bool_false'")


def test_bool_bool_true(db, vsql_data):
	assert not expr(db, "r.v_bool != True", where="r.identifier == 'bool_true'")


def test_bool_int_false(db, vsql_data):
	assert not expr(db, "r.v_bool != 0", where="r.identifier == 'bool_false'")


def test_bool_int_true(db, vsql_data):
	assert not expr(db, "r.v_bool != 1", where="r.identifier == 'bool_true'")


def test_int_bool_false(db, vsql_data):
	assert not expr(db, "0 != r.v_bool", where="r.identifier == 'bool_false'")


def test_int_bool_true(db, vsql_data):
	assert not expr(db, "1 != r.v_bool", where="r.identifier == 'bool_true'")


def test_number_bool_false(db, vsql_data):
	assert not expr(db, "0.0 != r.v_bool", where="r.identifier == 'bool_false'")


def test_number_bool_true(db, vsql_data):
	assert not expr(db, "1.0 != r.v_bool", where="r.identifier == 'bool_true'")


def test_number_int_false(db, vsql_data):
	assert expr(db, "42.5 != r.v_int", where="r.identifier == 'int'")


def test_number_int_true(db, vsql_data):
	assert not expr(db, "1776.0 != r.v_int", where="r.identifier == 'int'")


def test_number_number_false(db, vsql_data):
	assert expr(db, "17.23 != r.v_number", where="r.identifier == 'number'")


def test_number_number_true(db, vsql_data):
	assert not expr(db, "42.5 != r.v_number", where="r.identifier == 'number'")


def test_str_str_false(db, vsql_data):
	assert expr(db, "r.v_str != 'gurk'", where="r.identifier == 'none'")


def test_str_str_true(db, vsql_data):
	assert not expr(db, "r.v_str != 'gurk'", where="r.identifier == 'str'")


def test_date_date_false(db, vsql_data):
	assert expr(db, "r.v_date != @(2000-02-29)", where="r.identifier == 'none'")


def test_date_date_true(db, vsql_data):
	assert not expr(db, "r.v_date != @(2000-02-29)", where="r.identifier == 'date'")


def test_datetime_datetime_false(db, vsql_data):
	assert expr(db, "r.v_datetime != @(2000-02-29T12:34:56)", where="r.identifier == 'none'")


def test_datetime_datetime_true(db, vsql_data):
	assert not expr(db, "r.v_datetime != @(2000-02-29T12:34:56)", where="r.identifier == 'datetime'")


def test_datedelta_datedelta_false(db, vsql_data):
	assert expr(db, "r.v_datedelta != days(12)", where="r.identifier == 'none'")


def test_datedelta_datedelta_true(db, vsql_data):
	assert not expr(db, "r.v_datedelta != days(12)", where="r.identifier == 'datedelta'")


def test_color_color_false(db, vsql_data):
	assert expr(db, "r.v_color != #369c", where="r.identifier == 'none'")


def test_color_color_true(db, vsql_data):
	assert not expr(db, "r.v_color != #123456ff", where="r.identifier == 'color'")


def test_datetimedelta_datetimedelta_false(db, vsql_data):
	assert expr(db, "r.v_datetimedelta != timedelta(1, 45296)", where="r.identifier == 'none'")


def test_datetimedelta_datetimedelta_true(db, vsql_data):
	assert not expr(db, "r.v_datetimedelta != timedelta(1, 45296)", where="r.identifier == 'datetimedelta'")


def test_intlist_intlist1(db, vsql_data):
	assert expr(db, "[1] != [2]")


def test_intlist_intlist2(db, vsql_data):
	assert expr(db, "[1] != [1, 2]")


def test_intlist_intlist3(db, vsql_data):
	assert expr(db, "[1, 2] != [1]")


def test_intlist_intlist4(db, vsql_data):
	assert expr(db, "[1, None] != [1]")


def test_intlist_intlist5(db, vsql_data):
	assert not expr(db, "[1, None, 2, None, 3] != [1, None, 2, None, 3]")


def test_intlist_numberlist1(db, vsql_data):
	assert expr(db, "[1] != [1.5]")


def test_intlist_numberlist2(db, vsql_data):
	assert expr(db, "[1] != [1.0, 2.0]")


def test_intlist_numberlist3(db, vsql_data):
	assert expr(db, "[1, 2] != [1.0]")


def test_intlist_numberlist4(db, vsql_data):
	assert expr(db, "[1, None] != [1.0]")


def test_intlist_numberlist5(db, vsql_data):
	assert not expr(db, "[1, None, 2, None, 3] != [1.0, None, 2.0, None, 3.0]")


def test_numberlist_intlist1(db, vsql_data):
	assert expr(db, "[1.5] != [2]")


def test_numberlist_intlist2(db, vsql_data):
	assert expr(db, "[1.0] != [1, 2]")


def test_numberlist_intlist3(db, vsql_data):
	assert expr(db, "[1.0, 2.0] != [1]")


def test_numberlist_intlist4(db, vsql_data):
	assert expr(db, "[1.0, None] != [1]")


def test_numberlist_intlist5(db, vsql_data):
	assert not expr(db, "[1.0, None, 2.0, None, 3.0] != [1, None, 2, None, 3]")


def test_numberlist_numberlist1(db, vsql_data):
	assert expr(db, "[1.5] != [2.5]")


def test_numberlist_numberlist2(db, vsql_data):
	assert expr(db, "[1.0] != [1.0, 2.0]")


def test_numberlist_numberlist3(db, vsql_data):
	assert expr(db, "[1.0, 2.0] != [1.0]")


def test_numberlist_numberlist4(db, vsql_data):
	assert expr(db, "[1.5, None] != [1.5]")


def test_numberlist_numberlist5(db, vsql_data):
	assert not expr(db, "[1.5, None, 2.5, None, 3.5] != [1.5, None, 2.5, None, 3.5]")


def test_strlist_strlist1(db, vsql_data):
	assert expr(db, "['foo'] != ['bar']")


def test_strlist_strlist2(db, vsql_data):
	assert expr(db, "['foo'] != ['foo', 'bar']")


def test_strlist_strlist3(db, vsql_data):
	assert expr(db, "['foo', 'bar'] != ['foo']")


def test_strlist_strlist4(db, vsql_data):
	assert expr(db, "['foo', None] != ['foo']")


def test_strlist_strlist5(db, vsql_data):
	assert not expr(db, "['foo', None, 'bar', None, 'baz'] != ['foo', None, 'bar', None, 'baz']")


def test_datelist_datelist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29)] != [@(2000-03-01)]")


def test_datelist_datelist2(db, vsql_data):
	assert expr(db, "[@(2000-02-29)] != [@(2000-02-29), @(2000-03-01)]")


def test_datelist_datelist3(db, vsql_data):
	assert expr(db, "[@(2000-02-29), @(2000-03-01)] != [@(2000-02-29)]")


def test_datelist_datelist4(db, vsql_data):
	assert expr(db, "[@(2000-02-29), None] != [@(2000-02-29)]")


def test_datelist_datelist5(db, vsql_data):
	assert not expr(db, "[@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)] != [@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)]")


def test_datetimelist_datetimelist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56)] != [@(2000-03-01T12:34:56)]")


def test_datetimelist_datetimelist2(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56)] != [@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)]")


def test_datetimelist_datetimelist3(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)] != [@(2000-02-29T12:34:56)]")


def test_datetimelist_datetimelist4(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), None] != [@(2000-02-29T12:34:56)]")


def test_datetimelist_datetimelist5(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)] != [@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)]")


def test_nulllist_nulllist1(db, vsql_data):
	assert not expr(db, "[] != []")


def test_nulllist_nulllist2(db, vsql_data):
	assert not expr(db, "[None, None] != [None, None]")


def test_nulllist_nulllist3(db, vsql_data):
	assert expr(db, "[None] != [None, None]")


def test_nulllist_intlist1(db, vsql_data):
	assert expr(db, "[] != [1]")


def test_nulllist_intlist2(db, vsql_data):
	assert expr(db, "[None] != [1]")


def test_nulllist_intlist3(db, vsql_data):
	assert expr(db, "[None, None] != [1]")


def test_nulllist_numberlist1(db, vsql_data):
	assert expr(db, "[] != [1.1]")


def test_nulllist_numberlist2(db, vsql_data):
	assert expr(db, "[None] != [1.1]")


def test_nulllist_numberlist3(db, vsql_data):
	assert expr(db, "[None, None] != [1.1]")


def test_nulllist_strlist1(db, vsql_data):
	assert expr(db, "[] != ['gurk']")


def test_nulllist_strlist2(db, vsql_data):
	assert expr(db, "[None] != ['gurk']")


def test_nulllist_strlist3(db, vsql_data):
	assert expr(db, "[None, None] != ['gurk']")


def test_nulllist_datelist1(db, vsql_data):
	assert expr(db, "[] != [@(2000-02-29)]")


def test_nulllist_datelist2(db, vsql_data):
	assert expr(db, "[None] != [@(2000-02-29)]")


def test_nulllist_datelist3(db, vsql_data):
	assert expr(db, "[None, None] != [@(2000-02-29)]")


def test_nulllist_datetimelist1(db, vsql_data):
	assert expr(db, "[] != [@(2000-02-29T12:34:56)]")


def test_nulllist_datetimelist2(db, vsql_data):
	assert expr(db, "[None] != [@(2000-02-29T12:34:56)]")


def test_nulllist_datetimelist3(db, vsql_data):
	assert expr(db, "[None, None] != [@(2000-02-29T12:34:56)]")


def test_intlist_nulllist1(db, vsql_data):
	assert expr(db, "[1] != []")


def test_intlist_nulllist2(db, vsql_data):
	assert expr(db, "[1] != [None]")


def test_intlist_nulllist3(db, vsql_data):
	assert expr(db, "[1] != [None, None]")


def test_numberlist_nulllist1(db, vsql_data):
	assert expr(db, "[1.1] != []")


def test_numberlist_nulllist2(db, vsql_data):
	assert expr(db, "[1.1] != [None]")


def test_numberlist_nulllist3(db, vsql_data):
	assert expr(db, "[1.1] != [None, None]")


def test_strlist_nulllist1(db, vsql_data):
	assert expr(db, "['gurk'] != []")


def test_strlist_nulllist2(db, vsql_data):
	assert expr(db, "['gurk'] != [None]")


def test_strlist_nulllist3(db, vsql_data):
	assert expr(db, "['gurk'] != [None, None]")


def test_datelist_nulllist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29)] != []")


def test_datelist_nulllist2(db, vsql_data):
	assert expr(db, "[@(2000-02-29)] != [None]")


def test_datelist_nulllist3(db, vsql_data):
	assert expr(db, "[@(2000-02-29)] != [None, None]")


def test_datetimelist_nulllist1(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56)] != []")


def test_datetimelist_nulllist2(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56)] != [None]")


def test_datetimelist_nulllist3(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56)] != [None, None]")


def test_intset_intset1(db, vsql_data):
	assert expr(db, "{1} != {2}")


def test_intset_intset2(db, vsql_data):
	assert expr(db, "{1} != {1, 2}")


def test_intset_intset3(db, vsql_data):
	assert expr(db, "{1, 2} != {1}")


def test_intset_intset4(db, vsql_data):
	assert not expr(db, "{1, 2} != {2, 1}")


def test_intset_intset5(db, vsql_data):
	assert expr(db, "{1, None} != {1}")


def test_intset_intset6(db, vsql_data):
	assert not expr(db, "{1, None, 2, None, 3} != {None, 3, 2, 1, None}")


def test_numberset_numberset1(db, vsql_data):
	assert expr(db, "{1.5} != {2.5}")


def test_numberset_numberset2(db, vsql_data):
	assert expr(db, "{1.5} != {1.5, 2.5}")


def test_numberset_numberset3(db, vsql_data):
	assert expr(db, "{1.5, 2.5} != {1.5}")


def test_numberset_numberset4(db, vsql_data):
	assert not expr(db, "{1.5, 2.5} != {2.5, 1.5}")


def test_numberset_numberset5(db, vsql_data):
	assert expr(db, "{1.5, None} != {1.5}")


def test_numberset_numberset6(db, vsql_data):
	assert not expr(db, "{1.5, None, 2.5, None, 3.5} != {None, 3.5, 2.5, 1.5, None}")


def test_strset_strset1(db, vsql_data):
	assert expr(db, "{1.5} != {2.5}")


def test_strset_strset2(db, vsql_data):
	assert expr(db, "{'foo'} != {'foo', 'bar'}")


def test_strset_strset3(db, vsql_data):
	assert expr(db, "{'foo', 'bar'} != {'foo'}")


def test_strset_strset4(db, vsql_data):
	assert not expr(db, "{'foo', 'bar'} != {'bar', 'foo'}")


def test_strset_strset5(db, vsql_data):
	assert expr(db, "{'foo', None} != {'foo'}")


def test_strset_strset6(db, vsql_data):
	assert not expr(db, "{'foo', None, 'bar', None, 'baz'} != {None, 'baz', 'bar', 'foo', None}")


def test_dateset_dateset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29)} != {@(2000-03-01)}")


def test_dateset_dateset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29)} != {@(2000-02-29), @(2000-03-01)}")


def test_dateset_dateset3(db, vsql_data):
	assert expr(db, "{@(2000-02-29), @(2000-03-01)} != {@(2000-02-29)}")


def test_dateset_dateset4(db, vsql_data):
	assert not expr(db, "{@(2000-02-29), @(2000-03-01)} != {@(2000-03-01), @(2000-02-29)}")


def test_dateset_dateset5(db, vsql_data):
	assert expr(db, "{@(2000-02-29), None} != {@(2000-02-29)}")


def test_dateset_dateset6(db, vsql_data):
	assert not expr(db, "{@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)} != {None, @(2000-03-02), @(2000-03-01), @(2000-02-29), None}")


def test_datetimeset_datetimeset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56)} != {@(2000-03-01T12:34:56)}")


def test_datetimeset_datetimeset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56)} != {@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)}")


def test_datetimeset_datetimeset3(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} != {@(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset4(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} != {@(2000-03-01T12:34:56), @(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset5(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), None} != {@(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset6(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)} != {None, @(2000-03-02T12:34:56), @(2000-03-01T12:34:56), @(2000-02-29T12:34:56), None}")


def test_nullset_nullset1(db, vsql_data):
	assert not expr(db, "{/} != {/}")


def test_nullset_nullset2(db, vsql_data):
	assert not expr(db, "{None} != {None}")


def test_nullset_nullset3(db, vsql_data):
	assert not expr(db, "{None} != {None, None}")


def test_nullset_nullset4(db, vsql_data):
	assert expr(db, "{/} != {None, None}")


def test_nullset_intset1(db, vsql_data):
	assert expr(db, "{/} != {1}")


def test_nullset_intset2(db, vsql_data):
	assert expr(db, "{None} != {1}")


def test_nullset_intset3(db, vsql_data):
	assert expr(db, "{None, None} != {1}")


def test_nullset_intset4(db, vsql_data):
	assert expr(db, "{None, None} != {1, 2}")


def test_nullset_numberset1(db, vsql_data):
	assert expr(db, "{/} != {1.1}")


def test_nullset_numberset2(db, vsql_data):
	assert expr(db, "{None} != {1.1}")


def test_nullset_numberset3(db, vsql_data):
	assert expr(db, "{None, None} != {1.1}")


def test_nullset_numberset4(db, vsql_data):
	assert expr(db, "{None, None} != {1.1, 2.2}")


def test_nullset_strset1(db, vsql_data):
	assert expr(db, "{/} != {'gurk'}")


def test_nullset_strset2(db, vsql_data):
	assert expr(db, "{None} != {'gurk'}")


def test_nullset_strset3(db, vsql_data):
	assert expr(db, "{None, None} != {'gurk'}")


def test_nullset_strset4(db, vsql_data):
	assert expr(db, "{None, None} != {'gurk', 'hurz'}")


def test_nullset_dateset1(db, vsql_data):
	assert expr(db, "{/} != {@(2000-02-29)}")


def test_nullset_dateset2(db, vsql_data):
	assert expr(db, "{None} != {@(2000-02-29)}")


def test_nullset_dateset3(db, vsql_data):
	assert expr(db, "{None, None} != {@(2000-02-29)}")


def test_nullset_dateset4(db, vsql_data):
	assert expr(db, "{None, None} != {@(2000-02-29), @(2000-03-01)}")


def test_nullset_datetimeset1(db, vsql_data):
	assert expr(db, "{/} != {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset2(db, vsql_data):
	assert expr(db, "{None} != {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset3(db, vsql_data):
	assert expr(db, "{None, None} != {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset4(db, vsql_data):
	assert expr(db, "{None, None} != {@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)}")


def test_intset_nullset1(db, vsql_data):
	assert expr(db, "{1} != {/}")


def test_intset_nullset2(db, vsql_data):
	assert expr(db, "{1} != {None}")


def test_intset_nullset3(db, vsql_data):
	assert expr(db, "{1} != {None, None}")


def test_intset_nullset4(db, vsql_data):
	assert expr(db, "{1, 2} != {None, None}")


def test_numberset_nullset1(db, vsql_data):
	assert expr(db, "{1.1} != {/}")


def test_numberset_nullset2(db, vsql_data):
	assert expr(db, "{1.1} != {None}")


def test_numberset_nullset3(db, vsql_data):
	assert expr(db, "{1.1} != {None, None}")


def test_numberset_nullset4(db, vsql_data):
	assert expr(db, "{1.1, 2.2} != {None, None}")


def test_strset_nullset1(db, vsql_data):
	assert expr(db, "{'gurk'} != {/}")


def test_strset_nullset2(db, vsql_data):
	assert expr(db, "{'gurk'} != {None}")


def test_strset_nullset3(db, vsql_data):
	assert expr(db, "{'gurk'} != {None, None}")


def test_strset_nullset4(db, vsql_data):
	assert expr(db, "{'gurk', 'hurz'} != {None, None}")


def test_dateset_nullset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29)} != {/}")


def test_dateset_nullset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29)} != {None}")


def test_dateset_nullset3(db, vsql_data):
	assert expr(db, "{@(2000-02-29)} != {None, None}")


def test_dateset_nullset4(db, vsql_data):
	assert expr(db, "{@(2000-02-29), @(2000-03-01)} != {None, None}")


def test_datetimeset_nullset1(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56)} != {/}")


def test_datetimeset_nullset2(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56)} != {None}")


def test_datetimeset_nullset3(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56)} != {None, None}")


def test_datetimeset_nullset4(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} != {None, None}")


# FIXME Add tests for mixed type comparisons?


"""
Tests for the vSQL floor division ``//``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool_bool1(db, vsql_data):
	assert expr(db, "r.v_int // True", where="r.identifier == 'none'") is None


def test_bool_bool2(db, vsql_data):
	assert expr(db, "r.v_bool // True", where="r.identifier == 'bool_false'") == 0


def test_bool_bool3(db, vsql_data):
	assert expr(db, "r.v_bool // True", where="r.identifier == 'bool_true'") == 1


def test_bool_int(db, vsql_data):
	assert expr(db, "r.v_bool // 1", where="r.identifier == 'bool_true'") == 1


def test_bool_number(db, vsql_data):
	assert expr(db, "r.v_bool // 0.5", where="r.identifier == 'bool_true'") == 2


def test_int_bool(db, vsql_data):
	assert expr(db, "2 // r.v_bool", where="r.identifier == 'bool_true'") == 2


def test_int_int(db, vsql_data):
	assert expr(db, "r.v_int // 2", where="r.identifier == 'int'") == 888


def test_int_number(db, vsql_data):
	assert expr(db, "85 // r.v_number", where="r.identifier == 'number'") == 2


def test_number_bool(db, vsql_data):
	assert expr(db, "r.v_number // True", where="r.identifier == 'number'") == 42


def test_number_int(db, vsql_data):
	assert expr(db, "r.v_number // 2", where="r.identifier == 'number'") == 21


def test_number_number(db, vsql_data):
	assert expr(db, "r.v_number // 3.5", where="r.identifier == 'number'") == 12


def test_datedelta_bool(db, vsql_data):
	assert expr(db, "r.v_datedelta // True", where="r.identifier == 'datedelta'") == 12


def test_datedelta_int(db, vsql_data):
	assert expr(db, "r.v_datedelta // 5", where="r.identifier == 'datedelta'") == 2


def test_monthdelta_bool(db, vsql_data):
	assert expr(db, "r.v_monthdelta // True", where="r.identifier == 'monthdelta'") == 3


def test_monthdelta_int(db, vsql_data):
	assert expr(db, "r.v_monthdelta // 2", where="r.identifier == 'monthdelta'") == 1


def test_datetimedelta_bool(db, vsql_data):
	assert expr(db, "r.v_datetimedelta // True", where="r.identifier == 'datetimedelta'") == 1


def test_datetimedelta_int(db, vsql_data):
	assert expr(db, "r.v_datetimedelta // 2", where="r.identifier == 'datetimedelta'") == 0


def test_datetimedelta_number(db, vsql_data):
	assert expr(db, "r.v_datetimedelta // 12.5", where="r.identifier == 'datetimedelta'") == 0


"""
Tests for the vSQL equality comparision operator ``==``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import expr


###
### Tests
###

def test_bool_none_false(db, vsql_data):
	assert expr(db, "r.v_bool == None", where="r.identifier == 'none'")


def test_bool_none_true(db, vsql_data):
	assert not expr(db, "r.v_bool == None", where="r.identifier == 'bool_true'")


def test_int_none_false(db, vsql_data):
	assert expr(db, "r.v_int == None", where="r.identifier == 'none'")


def test_int_none_true(db, vsql_data):
	assert not expr(db, "r.v_int == None", where="r.identifier == 'int'")


def test_number_none_false(db, vsql_data):
	assert expr(db, "r.v_number == None", where="r.identifier == 'none'")


def test_number_none_true(db, vsql_data):
	assert not expr(db, "r.v_number == None", where="r.identifier == 'number'")


def test_str_none_false(db, vsql_data):
	assert expr(db, "r.v_str == None", where="r.identifier == 'none'")


def test_str_none_true(db, vsql_data):
	assert not expr(db, "r.v_str == None", where="r.identifier == 'str'")


def test_date_none_false(db, vsql_data):
	assert expr(db, "r.v_date == None", where="r.identifier == 'none'")


def test_date_none_true(db, vsql_data):
	assert not expr(db, "r.v_date == None", where="r.identifier == 'date'")


def test_datetime_none_false(db, vsql_data):
	assert expr(db, "r.v_datetime == None", where="r.identifier == 'none'")


def test_datetime_none_true(db, vsql_data):
	assert not expr(db, "r.v_datetime == None", where="r.identifier == 'datetime'")


def test_color_none_false(db, vsql_data):
	assert expr(db, "r.v_color == None", where="r.identifier == 'none'")


def test_color_none_true(db, vsql_data):
	assert not expr(db, "r.v_color == None", where="r.identifier == 'color'")


def test_datedelta_none_false(db, vsql_data):
	assert expr(db, "r.v_datedelta == None", where="r.identifier == 'none'")


def test_datedelta_none_true(db, vsql_data):
	assert not expr(db, "r.v_datedelta == None", where="r.identifier == 'datedelta'")


def test_datetimedelta_none_false(db, vsql_data):
	assert expr(db, "r.v_datetimedelta == None", where="r.identifier == 'none'")


def test_datetimedelta_none_true(db, vsql_data):
	assert not expr(db, "r.v_datetimedelta == None", where="r.identifier == 'datetimedelta'")


def test_monthdelta_none_false(db, vsql_data):
	assert expr(db, "r.v_monthdelta == None", where="r.identifier == 'none'")


def test_monthdelta_none_true(db, vsql_data):
	assert not expr(db, "r.v_monthdelta == None", where="r.identifier == 'monthdelta'")


def test_bool_bool_false(db, vsql_data):
	assert expr(db, "r.v_bool == False", where="r.identifier == 'bool_false'")


def test_bool_bool_true(db, vsql_data):
	assert expr(db, "r.v_bool == True", where="r.identifier == 'bool_true'")


def test_bool_int_false(db, vsql_data):
	assert expr(db, "r.v_bool == 0", where="r.identifier == 'bool_false'")


def test_bool_int_true(db, vsql_data):
	assert expr(db, "r.v_bool == 1", where="r.identifier == 'bool_true'")


def test_int_bool_false(db, vsql_data):
	assert expr(db, "0 == r.v_bool", where="r.identifier == 'bool_false'")


def test_int_bool_true(db, vsql_data):
	assert expr(db, "1 == r.v_bool", where="r.identifier == 'bool_true'")


def test_number_bool_false(db, vsql_data):
	assert expr(db, "0.0 == r.v_bool", where="r.identifier == 'bool_false'")


def test_number_bool_true(db, vsql_data):
	assert expr(db, "1.0 == r.v_bool", where="r.identifier == 'bool_true'")


def test_number_int_false(db, vsql_data):
	assert not expr(db, "42.5 == r.v_int", where="r.identifier == 'int'")


def test_number_int_true(db, vsql_data):
	assert expr(db, "1776.0 == r.v_int", where="r.identifier == 'int'")


def test_number_number_false(db, vsql_data):
	assert not expr(db, "17.23 == r.v_number", where="r.identifier == 'number'")


def test_number_number_true(db, vsql_data):
	assert expr(db, "42.5 == r.v_number", where="r.identifier == 'number'")


def test_str_str_false(db, vsql_data):
	assert not expr(db, "r.v_str == 'gurk'", where="r.identifier == 'none'")


def test_str_str_true(db, vsql_data):
	assert expr(db, "r.v_str == 'gurk'", where="r.identifier == 'str'")


def test_date_date_false(db, vsql_data):
	assert not expr(db, "r.v_date == @(2000-02-29)", where="r.identifier == 'none'")


def test_date_date_true(db, vsql_data):
	assert expr(db, "r.v_date == @(2000-02-29)", where="r.identifier == 'date'")


def test_datetime_datetime_false(db, vsql_data):
	assert not expr(db, "r.v_datetime == @(2000-02-29T12:34:56)", where="r.identifier == 'none'")


def test_datetime_datetime_true(db, vsql_data):
	assert expr(db, "r.v_datetime == @(2000-02-29T12:34:56)", where="r.identifier == 'datetime'")


def test_datedelta_datedelta_false(db, vsql_data):
	assert not expr(db, "r.v_datedelta == days(12)", where="r.identifier == 'none'")


def test_datedelta_datedelta_true(db, vsql_data):
	assert expr(db, "r.v_datedelta == days(12)", where="r.identifier == 'datedelta'")


def test_color_color_false(db, vsql_data):
	assert not expr(db, "r.v_color == #369c", where="r.identifier == 'none'")


def test_color_color_true(db, vsql_data):
	assert expr(db, "r.v_color == #123456ff", where="r.identifier == 'color'")


def test_datetimedelta_datetimedelta_false(db, vsql_data):
	assert not expr(db, "r.v_datetimedelta == timedelta(1, 45296)", where="r.identifier == 'none'")


def test_datetimedelta_datetimedelta_true(db, vsql_data):
	assert expr(db, "r.v_datetimedelta == timedelta(1, 45296)", where="r.identifier == 'datetimedelta'")


def test_intlist_intlist1(db, vsql_data):
	assert not expr(db, "[1] == [2]")


def test_intlist_intlist2(db, vsql_data):
	assert not expr(db, "[1] == [1, 2]")


def test_intlist_intlist3(db, vsql_data):
	assert not expr(db, "[1, 2] == [1]")


def test_intlist_intlist4(db, vsql_data):
	assert not expr(db, "[1, None] == [1]")


def test_intlist_intlist5(db, vsql_data):
	assert expr(db, "[1, None, 2, None, 3] == [1, None, 2, None, 3]")


def test_intlist_numberlist1(db, vsql_data):
	assert not expr(db, "[1] == [1.5]")


def test_intlist_numberlist2(db, vsql_data):
	assert not expr(db, "[1] == [1.0, 2.0]")


def test_intlist_numberlist3(db, vsql_data):
	assert not expr(db, "[1, 2] == [1.0]")


def test_intlist_numberlist4(db, vsql_data):
	assert not expr(db, "[1, None] == [1.0]")


def test_intlist_numberlist5(db, vsql_data):
	assert expr(db, "[1, None, 2, None, 3] == [1.0, None, 2.0, None, 3.0]")


def test_numberlist_intlist1(db, vsql_data):
	assert not expr(db, "[1.5] == [2]")


def test_numberlist_intlist2(db, vsql_data):
	assert not expr(db, "[1.0] == [1, 2]")


def test_numberlist_intlist3(db, vsql_data):
	assert not expr(db, "[1.0, 2.0] == [1]")


def test_numberlist_intlist4(db, vsql_data):
	assert not expr(db, "[1.0, None] == [1]")


def test_numberlist_intlist5(db, vsql_data):
	assert expr(db, "[1.0, None, 2.0, None, 3.0] == [1, None, 2, None, 3]")


def test_numberlist_numberlist1(db, vsql_data):
	assert not expr(db, "[1.5] == [2.5]")


def test_numberlist_numberlist2(db, vsql_data):
	assert not expr(db, "[1.0] == [1.0, 2.0]")


def test_numberlist_numberlist3(db, vsql_data):
	assert not expr(db, "[1.0, 2.0] == [1.0]")


def test_numberlist_numberlist4(db, vsql_data):
	assert not expr(db, "[1.5, None] == [1.5]")


def test_numberlist_numberlist5(db, vsql_data):
	assert expr(db, "[1.5, None, 2.5, None, 3.5] == [1.5, None, 2.5, None, 3.5]")


def test_strlist_strlist1(db, vsql_data):
	assert not expr(db, "['foo'] == ['bar']")


def test_strlist_strlist2(db, vsql_data):
	assert not expr(db, "['foo'] == ['foo', 'bar']")


def test_strlist_strlist3(db, vsql_data):
	assert not expr(db, "['foo', 'bar'] == ['foo']")


def test_strlist_strlist4(db, vsql_data):
	assert not expr(db, "['foo', None] == ['foo']")


def test_strlist_strlist5(db, vsql_data):
	assert expr(db, "['foo', None, 'bar', None, 'baz'] == ['foo', None, 'bar', None, 'baz']")


def test_datelist_datelist1(db, vsql_data):
	assert not expr(db, "[@(2000-02-29)] == [@(2000-03-01)]")


def test_datelist_datelist2(db, vsql_data):
	assert not expr(db, "[@(2000-02-29)] == [@(2000-02-29), @(2000-03-01)]")


def test_datelist_datelist3(db, vsql_data):
	assert not expr(db, "[@(2000-02-29), @(2000-03-01)] == [@(2000-02-29)]")


def test_datelist_datelist4(db, vsql_data):
	assert not expr(db, "[@(2000-02-29), None] == [@(2000-02-29)]")


def test_datelist_datelist5(db, vsql_data):
	assert expr(db, "[@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)] == [@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)]")


def test_datetimelist_datetimelist1(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56)] == [@(2000-03-01T12:34:56)]")


def test_datetimelist_datetimelist2(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56)] == [@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)]")


def test_datetimelist_datetimelist3(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)] == [@(2000-02-29T12:34:56)]")


def test_datetimelist_datetimelist4(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56), None] == [@(2000-02-29T12:34:56)]")


def test_datetimelist_datetimelist5(db, vsql_data):
	assert expr(db, "[@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)] == [@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)]")


def test_nulllist_nulllist1(db, vsql_data):
	assert expr(db, "[] == []")


def test_nulllist_nulllist2(db, vsql_data):
	assert expr(db, "[None, None] == [None, None]")


def test_nulllist_nulllist3(db, vsql_data):
	assert not expr(db, "[None] == [None, None]")


def test_nulllist_intlist1(db, vsql_data):
	assert not expr(db, "[] == [1]")


def test_nulllist_intlist2(db, vsql_data):
	assert not expr(db, "[None] == [1]")


def test_nulllist_intlist3(db, vsql_data):
	assert not expr(db, "[None, None] == [1]")


def test_nulllist_numberlist1(db, vsql_data):
	assert not expr(db, "[] == [1.1]")


def test_nulllist_numberlist2(db, vsql_data):
	assert not expr(db, "[None] == [1.1]")


def test_nulllist_numberlist3(db, vsql_data):
	assert not expr(db, "[None, None] == [1.1]")


def test_nulllist_strlist1(db, vsql_data):
	assert not expr(db, "[] == ['gurk']")


def test_nulllist_strlist2(db, vsql_data):
	assert not expr(db, "[None] == ['gurk']")


def test_nulllist_strlist3(db, vsql_data):
	assert not expr(db, "[None, None] == ['gurk']")


def test_nulllist_datelist1(db, vsql_data):
	assert not expr(db, "[] == [@(2000-02-29)]")


def test_nulllist_datelist2(db, vsql_data):
	assert not expr(db, "[None] == [@(2000-02-29)]")


def test_nulllist_datelist3(db, vsql_data):
	assert not expr(db, "[None, None] == [@(2000-02-29)]")


def test_nulllist_datetimelist1(db, vsql_data):
	assert not expr(db, "[] == [@(2000-02-29T12:34:56)]")


def test_nulllist_datetimelist2(db, vsql_data):
	assert not expr(db, "[None] == [@(2000-02-29T12:34:56)]")


def test_nulllist_datetimelist3(db, vsql_data):
	assert not expr(db, "[None, None] == [@(2000-02-29T12:34:56)]")


def test_intlist_nulllist1(db, vsql_data):
	assert not expr(db, "[1] == []")


def test_intlist_nulllist2(db, vsql_data):
	assert not expr(db, "[1] == [None]")


def test_intlist_nulllist3(db, vsql_data):
	assert not expr(db, "[1] == [None, None]")


def test_numberlist_nulllist1(db, vsql_data):
	assert not expr(db, "[1.1] == []")


def test_numberlist_nulllist2(db, vsql_data):
	assert not expr(db, "[1.1] == [None]")


def test_numberlist_nulllist3(db, vsql_data):
	assert not expr(db, "[1.1] == [None, None]")


def test_strlist_nulllist1(db, vsql_data):
	assert not expr(db, "['gurk'] == []")


def test_strlist_nulllist2(db, vsql_data):
	assert not expr(db, "['gurk'] == [None]")


def test_strlist_nulllist3(db, vsql_data):
	assert not expr(db, "['gurk'] == [None, None]")


def test_datelist_nulllist1(db, vsql_data):
	assert not expr(db, "[@(2000-02-29)] == []")


def test_datelist_nulllist2(db, vsql_data):
	assert not expr(db, "[@(2000-02-29)] == [None]")


def test_datelist_nulllist3(db, vsql_data):
	assert not expr(db, "[@(2000-02-29)] == [None, None]")


def test_datetimelist_nulllist1(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56)] == []")


def test_datetimelist_nulllist2(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56)] == [None]")


def test_datetimelist_nulllist3(db, vsql_data):
	assert not expr(db, "[@(2000-02-29T12:34:56)] == [None, None]")


def test_intset_intset1(db, vsql_data):
	assert not expr(db, "{1} == {2}")


def test_intset_intset2(db, vsql_data):
	assert not expr(db, "{1} == {1, 2}")


def test_intset_intset3(db, vsql_data):
	assert not expr(db, "{1, 2} == {1}")


def test_intset_intset4(db, vsql_data):
	assert expr(db, "({1, 2} == {2, 1})")


def test_intset_intset5(db, vsql_data):
	assert not expr(db, "{1, None} == {1}")


def test_intset_intset6(db, vsql_data):
	assert expr(db, "{1, None, 2, None, 3} == {None, 3, 2, 1, None}")


def test_numberset_numberset1(db, vsql_data):
	assert not expr(db, "{1.5} == {2.5}")


def test_numberset_numberset2(db, vsql_data):
	assert not expr(db, "{1.5} == {1.5, 2.5}")


def test_numberset_numberset3(db, vsql_data):
	assert not expr(db, "{1.5, 2.5} == {1.5}")


def test_numberset_numberset4(db, vsql_data):
	assert expr(db, "({1.5, 2.5} == {2.5, 1.5})")


def test_numberset_numberset5(db, vsql_data):
	assert not expr(db, "{1.5, None} == {1.5}")


def test_numberset_numberset6(db, vsql_data):
	assert expr(db, "{1.5, None, 2.5, None, 3.5} == {None, 3.5, 2.5, 1.5, None}")


def test_strset_strset1(db, vsql_data):
	assert not expr(db, "{1.5} == {2.5}")


def test_strset_strset2(db, vsql_data):
	assert not expr(db, "{'foo'} == {'foo', 'bar'}")


def test_strset_strset3(db, vsql_data):
	assert not expr(db, "{'foo', 'bar'} == {'foo'}")


def test_strset_strset4(db, vsql_data):
	assert expr(db, "{'foo', 'bar'} == {'bar', 'foo'}")


def test_strset_strset5(db, vsql_data):
	assert not expr(db, "{'foo', None} == {'foo'}")


def test_strset_strset6(db, vsql_data):
	assert expr(db, "{'foo', None, 'bar', None, 'baz'} == {None, 'baz', 'bar', 'foo', None}")


def test_dateset_dateset1(db, vsql_data):
	assert not expr(db, "{@(2000-02-29)} == {@(2000-03-01)}")


def test_dateset_dateset2(db, vsql_data):
	assert not expr(db, "{@(2000-02-29)} == {@(2000-02-29), @(2000-03-01)}")


def test_dateset_dateset3(db, vsql_data):
	assert not expr(db, "{@(2000-02-29), @(2000-03-01)} == {@(2000-02-29)}")


def test_dateset_dateset4(db, vsql_data):
	assert expr(db, "{@(2000-02-29), @(2000-03-01)} == {@(2000-03-01), @(2000-02-29)}")


def test_dateset_dateset5(db, vsql_data):
	assert not expr(db, "{@(2000-02-29), None} == {@(2000-02-29)}")


def test_dateset_dateset6(db, vsql_data):
	assert expr(db, "{@(2000-02-29), None, @(2000-03-01), None, @(2000-03-02)} == {None, @(2000-03-02), @(2000-03-01), @(2000-02-29), None}")


def test_datetimeset_datetimeset1(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56)} == {@(2000-03-01T12:34:56)}")


def test_datetimeset_datetimeset2(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56)} == {@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)}")


def test_datetimeset_datetimeset3(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} == {@(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset4(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} == {@(2000-03-01T12:34:56), @(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset5(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56), None} == {@(2000-02-29T12:34:56)}")


def test_datetimeset_datetimeset6(db, vsql_data):
	assert expr(db, "{@(2000-02-29T12:34:56), None, @(2000-03-01T12:34:56), None, @(2000-03-02T12:34:56)} == {None, @(2000-03-02T12:34:56), @(2000-03-01T12:34:56), @(2000-02-29T12:34:56), None}")


def test_nullset_nullset1(db, vsql_data):
	assert expr(db, "{/} == {/}")


def test_nullset_nullset2(db, vsql_data):
	assert expr(db, "{None} == {None}")


def test_nullset_nullset3(db, vsql_data):
	assert expr(db, "{None} == {None, None}")


def test_nullset_nullset4(db, vsql_data):
	assert not expr(db, "{/} == {None, None}")


def test_nullset_intset1(db, vsql_data):
	assert not expr(db, "{/} == {1}")


def test_nullset_intset2(db, vsql_data):
	assert not expr(db, "{None} == {1}")


def test_nullset_intset3(db, vsql_data):
	assert not expr(db, "{None, None} == {1}")


def test_nullset_intset4(db, vsql_data):
	assert not expr(db, "{None, None} == {1, 2}")


def test_nullset_numberset1(db, vsql_data):
	assert not expr(db, "{/} == {1.1}")


def test_nullset_numberset2(db, vsql_data):
	assert not expr(db, "{None} == {1.1}")


def test_nullset_numberset3(db, vsql_data):
	assert not expr(db, "{None, None} == {1.1}")


def test_nullset_numberset4(db, vsql_data):
	assert not expr(db, "{None, None} == {1.1, 2.2}")


def test_nullset_strset1(db, vsql_data):
	assert not expr(db, "{/} == {'gurk'}")


def test_nullset_strset2(db, vsql_data):
	assert not expr(db, "{None} == {'gurk'}")


def test_nullset_strset3(db, vsql_data):
	assert not expr(db, "{None, None} == {'gurk'}")


def test_nullset_strset4(db, vsql_data):
	assert not expr(db, "{None, None} == {'gurk', 'hurz'}")


def test_nullset_dateset1(db, vsql_data):
	assert not expr(db, "{/} == {@(2000-02-29)}")


def test_nullset_dateset2(db, vsql_data):
	assert not expr(db, "{None} == {@(2000-02-29)}")


def test_nullset_dateset3(db, vsql_data):
	assert not expr(db, "{None, None} == {@(2000-02-29)}")


def test_nullset_dateset4(db, vsql_data):
	assert not expr(db, "{None, None} == {@(2000-02-29), @(2000-03-01)}")


def test_nullset_datetimeset1(db, vsql_data):
	assert not expr(db, "{/} == {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset2(db, vsql_data):
	assert not expr(db, "{None} == {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset3(db, vsql_data):
	assert not expr(db, "{None, None} == {@(2000-02-29T12:34:56)}")


def test_nullset_datetimeset4(db, vsql_data):
	assert not expr(db, "{None, None} == {@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)}")


def test_intset_nullset1(db, vsql_data):
	assert not expr(db, "{1} == {/}")


def test_intset_nullset2(db, vsql_data):
	assert not expr(db, "{1} == {None}")


def test_intset_nullset3(db, vsql_data):
	assert not expr(db, "{1} == {None, None}")


def test_intset_nullset4(db, vsql_data):
	assert not expr(db, "{1, 2} == {None, None}")


def test_numberset_nullset1(db, vsql_data):
	assert not expr(db, "{1.1} == {/}")


def test_numberset_nullset2(db, vsql_data):
	assert not expr(db, "{1.1} == {None}")


def test_numberset_nullset3(db, vsql_data):
	assert not expr(db, "{1.1} == {None, None}")


def test_numberset_nullset4(db, vsql_data):
	assert not expr(db, "{1.1, 2.2} == {None, None}")


def test_strset_nullset1(db, vsql_data):
	assert not expr(db, "{'gurk'} == {/}")


def test_strset_nullset2(db, vsql_data):
	assert not expr(db, "{'gurk'} == {None}")


def test_strset_nullset3(db, vsql_data):
	assert not expr(db, "{'gurk'} == {None, None}")


def test_strset_nullset4(db, vsql_data):
	assert not expr(db, "{'gurk', 'hurz'} == {None, None}")


def test_dateset_nullset1(db, vsql_data):
	assert not expr(db, "{@(2000-02-29)} == {/}")


def test_dateset_nullset2(db, vsql_data):
	assert not expr(db, "{@(2000-02-29)} == {None}")


def test_dateset_nullset3(db, vsql_data):
	assert not expr(db, "{@(2000-02-29)} == {None, None}")


def test_dateset_nullset4(db, vsql_data):
	assert not expr(db, "{@(2000-02-29), @(2000-03-01)} == {None, None}")


def test_datetimeset_nullset1(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56)} == {/}")


def test_datetimeset_nullset2(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56)} == {None}")


def test_datetimeset_nullset3(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56)} == {None, None}")


def test_datetimeset_nullset4(db, vsql_data):
	assert not expr(db, "{@(2000-02-29T12:34:56), @(2000-03-01T12:34:56)} == {None, None}")


# FIXME Add tests for mixed type comparisons?


"""
Tests for the vSQL unary logical "not" operator ``not``.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool1(db, vsql_data):
	assert expr(db, "not r.v_bool", where="r.identifier == 'none'") == True


def test_bool2(db, vsql_data):
	assert expr(db, "not r.v_bool", where="r.identifier == 'bool_false'") == True


def test_bool3(db, vsql_data):
	assert expr(db, "not r.v_bool", where="r.identifier == 'bool_true'") == False


def test_int1(db, vsql_data):
	assert expr(db, "not r.v_int", where="r.identifier == 'none'") == True


def test_int2(db, vsql_data):
	assert expr(db, "not r.v_int", where="r.identifier == 'int'") == False


def test_number1(db, vsql_data):
	assert expr(db, "not r.v_number", where="r.identifier == 'none'") == True


def test_number2(db, vsql_data):
	assert expr(db, "not r.v_number", where="r.identifier == 'number'") == False


def test_str1(db, vsql_data):
	assert expr(db, "not r.v_str", where="r.identifier == 'none'") == True


def test_str2(db, vsql_data):
	assert expr(db, "not r.v_str", where="r.identifier == 'str'") == False


def test_date1(db, vsql_data):
	assert expr(db, "not r.v_date", where="r.identifier == 'none'") == True


def test_date2(db, vsql_data):
	assert expr(db, "not r.v_date", where="r.identifier == 'date'") == False


def test_datetime1(db, vsql_data):
	assert expr(db, "not r.v_datetime", where="r.identifier == 'none'") == True


def test_datetime2(db, vsql_data):
	assert expr(db, "not r.v_datetime", where="r.identifier == 'datetime'") == False


def test_datedelta1(db, vsql_data):
	assert expr(db, "not r.v_datedelta", where="r.identifier == 'none'") == True


def test_datedelta2(db, vsql_data):
	assert expr(db, "not r.v_datedelta", where="r.identifier == 'datedelta'") == False


def test_datetimedelta1(db, vsql_data):
	assert expr(db, "not r.v_datetimedelta", where="r.identifier == 'none'") == True


def test_datetimedelta2(db, vsql_data):
	assert expr(db, "not r.v_datetimedelta", where="r.identifier == 'datetimedelta'") == False


def test_monthdelta1(db, vsql_data):
	assert expr(db, "not r.v_monthdelta", where="r.identifier == 'none'") == True


def test_monthdelta2(db, vsql_data):
	assert expr(db, "not r.v_monthdelta", where="r.identifier == 'monthdelta'") == False


def test_color1(db, vsql_data):
	assert expr(db, "not r.v_color", where="r.identifier == 'none'") == True


def test_color2(db, vsql_data):
	assert expr(db, "not r.v_color", where="r.identifier == 'color'") == False


def test_geo(db, vsql_data):
	assert expr(db, "not geo(49, 11, 'Here')") == False
